"""Formula 6.29 from EN 1992-1-1:2004: Chapter 6 - Ultimate Limit State."""

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula
from blueprints.type_alias import NM, N
from blueprints.validations import raise_if_less_or_equal_to_zero, raise_if_negative


class Form6Dot29CheckTorsionShearResistance(Formula):
    r"""Class representing formula 6.29 for checking the maximum resistance of a member subjected to torsion and shear."""

    label = "6.29"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        t_ed: NM,
        v_ed: N,
        t_rd_max: NM,
        v_rd_max: N,
    ) -> None:
        r"""Check the maximum resistance of a member subjected to torsion and shear.

        EN 1992-1-1:2004 art.6.3.2(4) - Formula (6.29)

        Parameters
        ----------
        t_ed : NM
            [$T_{Ed}$] Design torsional moment [$Nm$].
        v_ed : N
            [$V_{Ed}$] Design transverse force [$N$].
        t_rd_max : NM
            [$T_{Rd,max}$] Design torsional resistance moment according to equation 6.30 [$Nm$].
        v_rd_max : N
            [$V_{Rd,max}$] Maximum design shear resistance according to Expressions (6.9) or (6.14) [$N$].
        """
        super().__init__()
        self.t_ed = t_ed
        self.v_ed = v_ed
        self.t_rd_max = t_rd_max
        self.v_rd_max = v_rd_max

    @staticmethod
    def _evaluate(
        t_ed: NM,
        v_ed: N,
        t_rd_max: NM,
        v_rd_max: N,
    ) -> bool:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_negative(t_ed=t_ed, v_ed=v_ed)
        raise_if_less_or_equal_to_zero(t_rd_max=t_rd_max, v_rd_max=v_rd_max)

        return (t_ed / t_rd_max + v_ed / v_rd_max) <= 1

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 6.29."""
        return LatexFormula(
            return_symbol=r"CHECK",
            result="OK" if self.__bool__() else "\\text{Not OK}",
            equation=r"\left( \frac{T_{Ed}}{T_{Rd,max}} + \frac{V_{Ed}}{V_{Rd,max}} \leq 1 \right)",
            numeric_equation=rf"\left( \frac{{{self.t_ed:.{n}f}}}{{{self.t_rd_max:.{n}f}}} + "
            rf"\frac{{{self.v_ed:.{n}f}}}{{{self.v_rd_max:.{n}f}}} \leq 1 \right)",
            comparison_operator_label="\\to",
            unit="",
        )



"""Formula 5.19 from EN 1992-1-1:2004: Chapter 5 - Structural Analysis."""

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula
from blueprints.type_alias import DIMENSIONLESS, KNM
from blueprints.validations import raise_if_less_or_equal_to_zero


class Form5Dot19EffectiveCreepCoefficient(Formula):
    """Class representing formula 5.19 for the calculation of the effective creep coefficient, ϕef."""

    label = "5.19"
    source_document = EN_1992_1_1_2004

    def __init__(self, phi_inf_t0: DIMENSIONLESS, m0_eqp: KNM, m0_ed: KNM) -> None:
        r"""[$\phi_{ef}$] Effective creep coefficient.

        EN 1992-1-1:2004 art.5.8.4(2) - Formula (5.19)

        Parameters
        ----------
        phi_inf_t0 : DIMENSIONLESS
            [$\phi (\infty,t_0)$] is the final value of the creep coefficient according to art. 3.1.4.
        m0_eqp : KNM
            [$M_{0,Eqp}$] is the first-order bending moment in the quasi-permanent load combination (SLS).
        m0_ed : KNM
            [$M_{0,Ed}$] is the first-order bending moment in the ultimate limit state (ULS).
        """
        super().__init__()
        self.phi_inf_t0 = phi_inf_t0
        self.m0_eqp = m0_eqp
        self.m0_ed = m0_ed

    @staticmethod
    def _evaluate(phi_inf_t0: DIMENSIONLESS, m0_eqp: KNM, m0_ed: KNM) -> DIMENSIONLESS:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_less_or_equal_to_zero(m0_ed=m0_ed)
        return phi_inf_t0 * (m0_eqp / m0_ed)

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 5.19."""
        return LatexFormula(
            return_symbol=r"\phi_{ef}",
            result=f"{self:.{n}f}",
            equation=r"\phi (\infty,t_0) \cdot \frac{M_{0,Eqp}}{M_{0,Ed}}",
            numeric_equation=rf"{self.phi_inf_t0} \cdot \frac{{{self.m0_eqp}}}{{{self.m0_ed}}}",
            comparison_operator_label="=",
        )



"""Rectangular reinforced cross-section."""

# ruff: noqa: PLR0913
from typing import Literal

from matplotlib import pyplot as plt
from shapely import LineString, Point, Polygon

from blueprints.materials.concrete import ConcreteMaterial
from blueprints.materials.reinforcement_steel import ReinforcementSteelMaterial
from blueprints.structural_sections.concrete.covers import CoversRectangular
from blueprints.structural_sections.concrete.reinforced_concrete_sections.base import ReinforcedCrossSection
from blueprints.structural_sections.concrete.reinforced_concrete_sections.plotters.rectangular import RectangularCrossSectionPlotter
from blueprints.structural_sections.concrete.reinforced_concrete_sections.reinforcement_configurations import ReinforcementByQuantity
from blueprints.structural_sections.concrete.stirrups import StirrupConfiguration
from blueprints.structural_sections.cross_section_rectangle import RectangularCrossSection
from blueprints.type_alias import DIMENSIONLESS, MM, RATIO


class RectangularReinforcedCrossSection(ReinforcedCrossSection):
    """Representation of a reinforced rectangular concrete cross-section like a beam.

    Parameters
    ----------
    width : MM
        The width of the rectangular cross-section [mm].
    height : MM
        The height of the rectangular cross-section [mm].
    concrete_material : ConcreteMaterial
        Material properties of the concrete.
    covers : CoversRectangular, optional
        The reinforcement covers for the cross-section [mm]. The default on all sides is 50 mm.
    name : str
        The name of the cross-section, default is "Rectangular Reinforced Concrete Section".
    """

    def __init__(
        self,
        width: MM,
        height: MM,
        concrete_material: ConcreteMaterial,
        covers: CoversRectangular = CoversRectangular(),
        name: str = "Rectangular Reinforced Concrete Section",
    ) -> None:
        """Initialize the rectangular reinforced concrete section."""
        super().__init__(
            cross_section=RectangularCrossSection(
                name="Rectangular Reinforced Concrete Section",
                width=width,
                height=height,
            ),
            concrete_material=concrete_material,
        )
        self.name = name
        self.width = width
        self.height = height
        self.covers = covers
        self.plotter = RectangularCrossSectionPlotter(cross_section=self)

    def add_stirrup_along_edges(
        self,
        diameter: MM,
        distance: MM,
        material: ReinforcementSteelMaterial,
        shear_check: bool = True,
        torsion_check: bool = True,
        mandrel_diameter_factor: DIMENSIONLESS | None = None,
        anchorage_length: MM = 0.0,
        relative_start_position: RATIO = 0.0,
        relative_end_position: RATIO = 1.0,
    ) -> StirrupConfiguration:
        """Adds a stirrup configuration along the edges of the cross-section taking the covers into account. The created configuration goes around
        the longitudinal rebars (if any).

        Use .add_stirrup_configuration() to add a stirrup configuration of any shape, size, and position (as long as it is inside the cross-section).


        Parameters
        ----------
        diameter: MM
            Diameter of the stirrups [mm].
        distance: MM
            Longitudinal distance between stirrups [mm].
        material : ReinforcementSteelMaterial
            Representation of the properties of reinforcement steel suitable for use with NEN-EN 1992-1-1
        shear_check: bool
            Take stirrup into account in shear check
        torsion_check: bool
            Take stirrup into account in torsion check
        mandrel_diameter_factor: DIMENSIONLESS
            Inner diameter of mandrel as multiple of stirrup diameter [-]
            (default: 4⌀ for ⌀<=16mm and 5⌀ for ⌀>16mm) Tabel 8.1Na NEN-EN 1992-1-1 Dutch National Annex.
        anchorage_length: MM
            Anchorage length [mm]
        relative_start_position: RATIO
            Relative position of the start of the stirrup configuration inside the cross-section (longitudinal direction). Value between 0 and 1.
            Default is 0 (start).
        relative_end_position: RATIO
            Relative position of the end of the stirrup configuration inside the cross-section (longitudinal direction). Value between 0 and 1.
            Default is 1 (end).


        Returns
        -------
        StirrupConfiguration
            Newly created stirrup configuration inside the cross-section.
        """
        # get the corners of the cross-section
        min_x, min_y, max_x, max_y = self.cross_section.polygon.bounds

        # create the corners of the stirrup configuration based on the covers present
        left_bottom_corner = Point(min_x + self.covers.left + (diameter / 2), min_y + self.covers.lower + (diameter / 2))
        left_top_corner = Point(min_x + self.covers.left + (diameter / 2), max_y - self.covers.upper - (diameter / 2))
        right_top_corner = Point(max_x - self.covers.right - (diameter / 2), max_y - self.covers.upper - (diameter / 2))
        right_bottom_corner = Point(max_x - self.covers.right - (diameter / 2), min_y + self.covers.lower + (diameter / 2))

        return self.add_stirrup_configuration(
            StirrupConfiguration(
                geometry=Polygon([left_bottom_corner, left_top_corner, right_top_corner, right_bottom_corner]),
                diameter=diameter,
                distance=distance,
                material=material,
                shear_check=shear_check,
                torsion_check=torsion_check,
                mandrel_diameter_factor=mandrel_diameter_factor,
                anchorage_length=anchorage_length,
                based_on_cover=True,
                relative_start_position=relative_start_position,
                relative_end_position=relative_end_position,
            )
        )

    def add_stirrup_in_center(
        self,
        width: MM,
        diameter: MM,
        distance: MM,
        material: ReinforcementSteelMaterial,
        shear_check: bool = True,
        torsion_check: bool = True,
        mandrel_diameter_factor: float | None = None,
        anchorage_length: MM = 0.0,
        relative_start_position: RATIO = 0.0,
        relative_end_position: RATIO = 1.0,
    ) -> StirrupConfiguration:
        """Add stirrups to the center of the reinforced cross-section based on a given width (ctc of the legs). The created configuration goes around
        the longitudinal rebars (if any).

        Use .add_stirrup_configuration() to add a stirrup configuration of any shape, size, and position (as long as it is inside the cross-section).

        Parameters
        ----------
        width: MM
            Total width of the stirrup taken from the center lines of the legs [mm].
        diameter: MM
            Diameter of the stirrups [mm].
        distance: MM
            Longitudinal distance between stirrups [mm].
        material : ReinforcementSteelMaterial
            Representation of the properties of reinforcement steel suitable for use with NEN-EN 1992-1-1
        shear_check: bool
            Take stirrup into account in shear check
        torsion_check: bool
            Take stirrup into account in torsion check
        mandrel_diameter_factor: DIMENSIONLESS
            Inner diameter of mandrel as multiple of stirrup diameter [-]
            (default: 4⌀ for ⌀<=16mm and 5⌀ for ⌀>16mm) Tabel 8.1Na NEN-EN 1992-1-1 Dutch National Annex.
        anchorage_length: MM
            Anchorage length [mm]
        relative_start_position: RATIO
            Relative position of the start of the stirrup configuration inside the cross-section (longitudinal direction). Value between 0 and 1.
            Default is 0 (start).
        relative_end_position: RATIO
            Relative position of the end of the stirrup configuration inside the cross-section (longitudinal direction). Value between 0 and 1.
            Default is 1 (end).

        Returns
        -------
        StirrupConfiguration
            Newly created stirrup configuration inside the cross-section.
        """
        # get the corners of the cross-section
        _, min_y, _, max_y = self.cross_section.polygon.bounds

        # create the corners of the stirrup configuration based on the covers present
        left_bottom_corner = Point(-width / 2, min_y + self.covers.lower + (diameter / 2))
        left_top_corner = Point(-width / 2, max_y - self.covers.upper - (diameter / 2))
        right_top_corner = Point(width / 2, max_y - self.covers.upper - (diameter / 2))
        right_bottom_corner = Point(width / 2, min_y + self.covers.lower + (diameter / 2))

        return self.add_stirrup_configuration(
            StirrupConfiguration(
                geometry=Polygon([left_bottom_corner, left_top_corner, right_top_corner, right_bottom_corner]),
                diameter=diameter,
                distance=distance,
                material=material,
                shear_check=shear_check,
                torsion_check=torsion_check,
                mandrel_diameter_factor=mandrel_diameter_factor,
                anchorage_length=anchorage_length,
                based_on_cover=True,
                relative_start_position=relative_start_position,
                relative_end_position=relative_end_position,
            )
        )

    def _get_reference_line(
        self,
        edge: Literal["upper", "right", "lower", "left"],
        diameter: MM,
        cover: MM | None = None,
        corner_offset: MM = 0.0,
    ) -> LineString:
        """Get the reference line for the given edge of the cross-section.

        Parameters
        ----------
        edge: Literal["upper", "right", "lower", "left"]
            Edge of the cross-section.
        diameter: MM
            Diameter of the rebars [mm].
        cover: MM, optional
            Cover of the rebars [mm]. If not provided, the default cover on the given edge of the cross-section is used.
        corner_offset: MM, optional
            The offset of the first and last rebars from the corners of the cross-section towards the center of the cross-section [mm]. If not
            provided, the rebars are to be placed at the corners taking into account the present covers and stirrups inside the cross-section.

        Returns
        -------
        LineString
            Reference line for the given edge of the cross-section.
        """
        # get cross-section corners
        min_x, min_y, max_x, max_y = self.cross_section.polygon.bounds

        # check if a custom cover is provided
        upper_cover = cover if cover is not None else self.covers.upper
        lower_cover = cover if cover is not None else self.covers.lower

        # check if there is a stirrup configuration present and adjust the cover
        max_stirrups_diameter = 0.0
        if self._stirrups:
            max_stirrups_diameter = max([stirrup.diameter for stirrup in self._stirrups])

        # define corner positions of a bar inside the cross-section
        upper_left = (
            min_x + self.covers.left + max_stirrups_diameter + corner_offset + diameter / 2,
            max_y - upper_cover - max_stirrups_diameter - diameter / 2,
        )
        upper_right = (
            max_x - self.covers.right - max_stirrups_diameter - corner_offset - diameter / 2,
            max_y - upper_cover - max_stirrups_diameter - diameter / 2,
        )
        lower_left = (
            min_x + self.covers.left + max_stirrups_diameter + corner_offset + diameter / 2,
            min_y + lower_cover + max_stirrups_diameter + diameter / 2,
        )
        lower_right = (
            max_x - self.covers.right - max_stirrups_diameter - corner_offset - diameter / 2,
            min_y + lower_cover + max_stirrups_diameter + diameter / 2,
        )

        match edge.lower():
            case "upper":
                start, end = upper_left, upper_right
            case "right":
                start, end = upper_right, lower_right
            case "lower":
                start, end = lower_left, lower_right
            case "left":
                start, end = upper_left, lower_left
            case _:  # pragma: no cover
                msg = f"Edge '{edge}' is not supported. Supported edges are 'upper', 'right', 'lower', and 'left'."
                raise ValueError(msg)

        return LineString([start, end])

    def add_longitudinal_reinforcement_by_quantity(
        self,
        n: int,
        diameter: MM,
        material: ReinforcementSteelMaterial,
        edge: Literal["upper", "right", "lower", "left"],
        cover: MM | None = None,
        corner_offset: MM = 0.0,
    ) -> None:
        """Add longitudinal reinforcement to the cross-section based on the quantity configuration of rebars and a given edge of the cross-section.

         for example: 5⌀12 on upper edge, 4⌀16 on lower edge, etc.

        Parameters
        ----------
        n: int
            Amount of longitudinal bars.
        diameter: MM
            Diameter of the rebars [mm].
        material : ReinforcementSteelMaterial
            Representation of the properties of reinforcement steel suitable for use with NEN-EN 1992-1-1.
        edge: Literal["upper", "right", "lower", "left"]
            Edge of the cross-section where the rebars are placed.
        cover: MM, optional
            Cover of the rebars [mm]. If not provided, the default cover on the given edge of the cross-section is used.
        corner_offset: MM, optional
            The offset of the first and last rebars from the corners of the cross-section towards the center of the cross-section [mm]. If not
            provided, the rebars are to be placed at the corners taking into account the present covers and stirrups inside the cross-section.
        """
        line = self._get_reference_line(
            edge=edge,
            diameter=diameter,
            cover=cover,
            corner_offset=corner_offset,
        )
        assert line
        return self.add_reinforcement_configuration(
            line=self._get_reference_line,
            configuration=ReinforcementByQuantity(
                diameter=diameter,
                material=material,
                n=n,
            ),
            edge=edge,
            cover=cover,
            corner_offset=corner_offset,
            diameter=diameter,
        )

    def plot(self, *args, **kwargs) -> plt.Figure:
        """Plot the cross-section. Making use of the standard plotter.

        If you want to use a custom plotter, use the .plotter attribute to plot the cross-section.

        Parameters
        ----------
        *args
            Additional arguments passed to the plotter.
        **kwargs
            Additional keyword arguments passed to the plotter.
        """
        return self.plotter.plot(*args, **kwargs)



"""Formula 5.7 from EN 1992-1-1:2004: Chapter 5 - Structural Analysis."""

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula
from blueprints.type_alias import M
from blueprints.validations import raise_if_negative


class Form5Dot7EffectiveFlangeWidth(Formula):
    """Class representing formula 5.7 for the calculation of effective flange width [$b_{eff}$] for a T beam or L beam.
    See Figure 5.3.
    """

    label = "5.7"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        *b_eff_i: M,
        b_w: M,
        b: M,
    ) -> None:
        r"""[$b_{eff}$] Effective flange width for a T beam or L beam [$m$].

        EN 1992-1-1:2004 art.5.3.2.1(3) - Formula (5.7)

        Parameters
        ----------
        b_eff_i : M
            [$b_{eff,i}$] Effective flange width of the i-th flange [$m$].
        b_w : M
            [$b_{w}$] Width of the web [$m$].
        b : M
            [$b$] Total width of the flange [$m$].

        Notes
        -----
        where [$b_{eff,i}$] is the effective flange width of the i-th flange
        """
        super().__init__()
        self.b_eff_i = b_eff_i
        self.b_w = b_w
        self.b = b

    @staticmethod
    def _evaluate(
        *b_eff_i: M,
        b_w: M,
        b: M,
    ) -> M:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_negative(
            **{f"b_eff_{i}": b_eff for i, b_eff in enumerate(b_eff_i)},
            b_w=b_w,
            b=b,
        )
        return min(sum(b_eff_i) + b_w, b)

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 5.7."""
        return LatexFormula(
            return_symbol=r"b_{eff}",
            result=f"{self:.{n}f}",
            equation=r"\sum b_{eff,i}+b_w\le b",
            numeric_equation=rf"\sum ({'+'.join(str(b_eff) for b_eff in self.b_eff_i)})+{self.b_w}\le {self.b}",
            comparison_operator_label="=",
        )



"""Formula 7.2 and 7.2sub1 from EN 1992-1-1:2004: Chapter 7 - Serviceability Limit State."""

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula, latex_replace_symbols
from blueprints.type_alias import DIMENSIONLESS, MM, MPA
from blueprints.validations import raise_if_negative


class Form7Dot2StressDistributionCoefficient(Formula):
    r"""Class representing formula 7.2 for the calculation of [$k_c$]."""

    label = "7.2"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        sigma_c: MPA,
        k_1: DIMENSIONLESS,
        h: MM,
        f_ct_eff: MPA,
    ) -> None:
        r"""[$k_c$] Calculation of the coefficient for stress distribution for bending or bending combined with axial forces.

        EN 1992-1-1:2004 art.7.3.2(2) - Formula (7.2)

        Parameters
        ----------
        sigma_c : MPA
            [$\sigma_c$] Compressive stress in the concrete, according to 7.4 [$MPa$].
        k_1 : DIMENSIONLESS
            [$k_1$] Coefficient considering the effects of axial forces on the stress distribution, according to 7.2sub1 [$-$].
        h : MM
            [$h$] Overall depth of the section [$mm$].
        f_ct_eff : MPA
            [$f_{ct,eff}$] Mean value of the tensile strength of the concrete effective at the time when the
            cracks may first be expected to occur [$MPa$].
        """
        super().__init__()
        self.sigma_c = sigma_c
        self.k_1 = k_1
        self.h = h
        self.f_ct_eff = f_ct_eff

    @staticmethod
    def _evaluate(
        sigma_c: MPA,
        k_1: DIMENSIONLESS,
        h: MM,
        f_ct_eff: MPA,
    ) -> DIMENSIONLESS:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_negative(sigma_c=sigma_c, k_1=k_1, h=h, f_ct_eff=f_ct_eff)

        h_star = h if h < 1000 else 1000
        return min(0.4 * (1 - (sigma_c / (k_1 * (h / h_star) * f_ct_eff))), 1)

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 7.2."""
        _equation: str = (
            r"min\left(0.4 \cdot \left(1 - \frac{\sigma_c}{k_1 \cdot \left(\frac{ h}{min( h, 1000)}\right) \cdot f_{ct,eff}}\right), 1\right)"
        )
        _numeric_equation: str = latex_replace_symbols(
            _equation,
            {
                r"\sigma_c": f"{self.sigma_c:.{n}f}",
                r"k_1": f"{self.k_1:.{n}f}",
                r" h": f" {self.h:.{n}f}",
                r"f_{ct,eff}": f"{self.f_ct_eff:.{n}f}",
            },
            False,
        )
        return LatexFormula(
            return_symbol=r"k_c",
            result=f"{self:.{n}f}",
            equation=_equation,
            numeric_equation=_numeric_equation,
            comparison_operator_label="=",
            unit="-",
        )


class Form7Dot2Sub1AxialForceCoefficient(Formula):
    r"""Class representing formula 7.2sub1 for the calculation of [$k_1$] factor."""

    label = "7.2sub1"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        n_ed: MPA,
        h: MM,
    ) -> None:
        r"""[$k_1$] Calculation of the coefficient for stress distribution for bending or bending combined with axial forces.

        EN 1992-1-1:2004 art.7.3.2(2) - Formula (7.2sub1)

        Parameters
        ----------
        n_ed : MPA
            [$N_{Ed}$] Axial force at the serviceability limit state acting on the part of the cross-section
            under consideration (compressive force positive). [$N_{Ed}$] should be determined considering the characteristic
            values of prestress and axial forces under the relevant combination of actions.
        h : MM
            [$h$] Overall depth of the section [$mm$].
        """
        super().__init__()
        self.n_ed = n_ed
        self.h = h

    @staticmethod
    def _evaluate(
        n_ed: MPA,
        h: MM,
    ) -> DIMENSIONLESS:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_negative(h=h)

        h_star = h if h < 1000 else 1000
        return 1.5 if n_ed > 0 else 2 * h_star / (3 * h)

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 7.2sub1."""
        _equation: str = r"\begin{cases} 1.5 & \text{if } N_{Ed} > 0 \\ \frac{2 \cdot min(h, 1000)}{3 \cdot h} & \text{if } N_{Ed} \le 0 \end{cases}"
        _numeric_equation: str = latex_replace_symbols(
            _equation,
            {
                "N_{Ed}": f"{self.n_ed:.{n}f}",
                "h": f"{self.h:.{n}f}",
            },
            False,
        )
        return LatexFormula(
            return_symbol=r"k_1",
            result=f"{self:.{n}f}",
            equation=_equation,
            numeric_equation=_numeric_equation,
            comparison_operator_label="=",
            unit="-",
        )



"""Testing formula 6.53 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_6_ultimate_limit_state.formula_6_53 import Form6Dot53CheckPunchingShear
from blueprints.validations import LessOrEqualToZeroError, NegativeValueError


class TestForm6Dot53CheckPunchingShear:
    """Validation for formula 6.53 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        beta = 1.1
        v_ed = 500.0
        u_0 = 400.0
        d = 200.0
        v_rd_max = 2.5

        # Object to test
        formula = Form6Dot53CheckPunchingShear(beta=beta, v_ed=v_ed, u_0=u_0, d=d, v_rd_max=v_rd_max)

        # Expected result, manually calculated
        expected_result = True

        assert formula == expected_result

    @pytest.mark.parametrize(
        ("beta", "v_ed", "u_0", "d", "v_rd_max"),
        [
            (-1.1, 500.0, 400.0, 200.0, 2.5),  # beta is negative
            (1.1, -500.0, 400.0, 200.0, 2.5),  # v_ed is negative
            (1.1, 500.0, -400.0, 200.0, 2.5),  # u_0 is negative
            (1.1, 500.0, 400.0, -200.0, 2.5),  # d is negative
            (1.1, 500.0, 400.0, 200.0, -2.5),  # v_rd_max is negative
            (1.1, 500.0, 0.0, 200.0, 2.5),  # u_0 is zero
            (1.1, 500.0, 400.0, 0.0, 2.5),  # d is zero
        ],
    )
    def test_raise_error_when_invalid_values_are_given(self, beta: float, v_ed: float, u_0: float, d: float, v_rd_max: float) -> None:
        """Test invalid values."""
        with pytest.raises((NegativeValueError, LessOrEqualToZeroError)):
            Form6Dot53CheckPunchingShear(beta=beta, v_ed=v_ed, u_0=u_0, d=d, v_rd_max=v_rd_max)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"CHECK \to \frac{\beta \cdot V_{Ed}}{u_{0} \cdot d} \leq v_{Rd,max} \to "
                r"\frac{1.100 \cdot 500.000}{400.000 \cdot 200.000} \leq 2.500 \to OK",
            ),
            ("short", r"CHECK \to OK"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        beta = 1.1
        v_ed = 500.0
        u_0 = 400.0
        d = 200.0
        v_rd_max = 2.5

        # Object to test
        latex = Form6Dot53CheckPunchingShear(beta=beta, v_ed=v_ed, u_0=u_0, d=d, v_rd_max=v_rd_max).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."



"""Formula 6.51 from EN 1992-1-1:2004: Chapter 6 - Ultimate Limit State."""

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula, latex_replace_symbols
from blueprints.type_alias import DIMENSIONLESS, MM, MM2, MPA, NMM, N
from blueprints.validations import raise_if_less_or_equal_to_zero, raise_if_negative


class Form6Dot51AppliedPunchingShearStressEccentricLoading(Formula):
    r"""Class representing formula 6.51 for the calculation of punching shear stress for eccentric loading [$v_{Ed}$]
    of slabs and column bases without shear reinforcement.
    """

    label = "6.51"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        v_ed_red: N,
        u: MM,
        d: MM,
        k: DIMENSIONLESS,
        m_ed: NMM,
        w: MM2,
    ) -> None:
        r"""[$v_{Ed}$] Calculation of punching shear stress for eccentric loading of slabs and column bases without shear reinforcement.

        EN 1992-1-1:2004 art.6.4.4(2) - Formula (6.51)

        Parameters
        ----------
        v_ed_red : N
            [$V_{Ed,red}$] Net applied punching force [$N$].
        u : MM
            [$u$] Perimeter of the critical section [$mm$].
        d : MM
            [$d$] Mean effective depth of the slab [$mm$].
        k : DIMENSIONLESS
            [$k$] Coefficient dependent on the ratio between the column dimensions as defined in 6.4.3(3) or 6.4.3(4) [$-$].
        m_ed : NMM
            [$M_{Ed}$] Design bending moment [$Nmm$].
        w : MM2
            [$W$] Similar to [$W_1$] as defined in 6.4.3(3) and 6.4.3.(4) but for perimeter [$u$] [$mm^2$].
        """
        super().__init__()
        self.v_ed_red = v_ed_red
        self.u = u
        self.d = d
        self.k = k
        self.m_ed = m_ed
        self.w = w

    @staticmethod
    def _evaluate(
        v_ed_red: N,
        u: MM,
        d: MM,
        k: DIMENSIONLESS,
        m_ed: NMM,
        w: MM2,
    ) -> MPA:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_negative(m_ed=m_ed)
        raise_if_less_or_equal_to_zero(v_ed_red=v_ed_red, u=u, d=d, k=k, w=w)

        return (v_ed_red / (u * d)) * (1 + k * (m_ed * u) / (v_ed_red * w))

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 6.51."""
        _equation: str = r"\frac{V_{Ed,red}}{u \cdot d} \cdot \left(1 + k \cdot \frac{M_{Ed} \cdot u}{V_{Ed,red} \cdot W}\right)"
        _numeric_equation: str = latex_replace_symbols(
            _equation,
            {
                r"V_{Ed,red}": f"{self.v_ed_red:.{n}f}",
                r"u": f"{self.u:.{n}f}",
                r" d": f" {self.d:.{n}f}",
                r"k": f"{self.k:.{n}f}",
                r"M_{Ed}": f"{self.m_ed:.{n}f}",
                r"W": f"{self.w:.{n}f}",
            },
            False,
        )
        return LatexFormula(
            return_symbol=r"v_{Ed}",
            result=f"{self:.{n}f}",
            equation=_equation,
            numeric_equation=_numeric_equation,
            comparison_operator_label="=",
            unit="MPa",
        )



"""Formula 6.16 from EN 1993-1-1:2005: Chapter 6 - Ultimate Limit State."""

from blueprints.codes.eurocode.en_1993_1_1_2005 import EN_1993_1_1_2005
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula, latex_replace_symbols
from blueprints.type_alias import DIMENSIONLESS, MM2, MPA
from blueprints.validations import raise_if_less_or_equal_to_zero, raise_if_negative


class Form6Dot16CheckFlangeWithFastenerHoles(Formula):
    r"""Class representing formula 6.16 for the test of the stresses where there are fastener holes in the tension flange."""

    label = "6.16"
    source_document = EN_1993_1_1_2005

    def __init__(
        self,
        a_f_net: MM2,
        f_u: MPA,
        gamma_m2: DIMENSIONLESS,
        a_f: MM2,
        f_y: MPA,
        gamma_m0: DIMENSIONLESS,
    ) -> None:
        r"""Test the stresses where there are fastener holes in the tension flange.

        EN 1993-1-1:2005 art.6.2.5(4) - Formula (6.16)

        Parameters
        ----------
        a_f_net : MM2
            [$A_{f,net}$] Net area of the flange with fastener holes [mm^2].
        f_u : MPA
            [$f_{u}$] Ultimate tensile strength of the material [MPa].
        gamma_m2 : DIMENSIONLESS
            [$\gamma_{M2}$] Partial safety factor for ultimate limit state [-].
        a_f : MM2
            [$A_{f}$] Gross area of the flange [mm^2].
        f_y : MPA
            [$f_{y}$] Yield strength of the material [MPa].
        gamma_m0 : DIMENSIONLESS
            [$\gamma_{M0}$] Partial safety factor for yield strength [-].
        """
        super().__init__()
        self.a_f_net = a_f_net
        self.f_u = f_u
        self.gamma_m2 = gamma_m2
        self.a_f = a_f
        self.f_y = f_y
        self.gamma_m0 = gamma_m0

    @staticmethod
    def _evaluate(
        a_f_net: MM2,
        f_u: MPA,
        gamma_m2: DIMENSIONLESS,
        a_f: MM2,
        f_y: MPA,
        gamma_m0: DIMENSIONLESS,
    ) -> bool:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_less_or_equal_to_zero(gamma_m2=gamma_m2, gamma_m0=gamma_m0)
        raise_if_negative(a_f_net=a_f_net, f_u=f_u, a_f=a_f, f_y=f_y)

        return (a_f_net * 0.9 * f_u / gamma_m2) >= (a_f * f_y / gamma_m0)

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 6.16."""
        _equation: str = r"\left( \frac{A_{f,net} \cdot 0.9 \cdot f_{u}}{\gamma_{M2}} \geq \frac{A_{f} \cdot f_{y}}{\gamma_{M0}} \right)"
        _numeric_equation: str = latex_replace_symbols(
            _equation,
            {
                r"A_{f,net}": f"{self.a_f_net:.{n}f}",
                r"f_{u}": f"{self.f_u:.{n}f}",
                r"\gamma_{M2}": f"{self.gamma_m2:.{n}f}",
                r"A_{f}": f"{self.a_f:.{n}f}",
                r"f_{y}": f"{self.f_y:.{n}f}",
                r"\gamma_{M0}": f"{self.gamma_m0:.{n}f}",
            },
            False,
        )

        return LatexFormula(
            return_symbol=r"CHECK",
            result="OK" if self.__bool__() else "\\text{Not OK}",
            equation=_equation,
            numeric_equation=_numeric_equation,
            comparison_operator_label="\\to",
            unit="",
        )



"""Testing formula 5.19 from EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_5_structural_analysis.formula_5_19 import Form5Dot19EffectiveCreepCoefficient
from blueprints.validations import LessOrEqualToZeroError


class TestForm5Dot19EffectiveCreepCoefficient:
    """Validation for formula 5.19 from EN 1992-1-1:2004."""

    @pytest.fixture
    def form_5_19(self) -> Form5Dot19EffectiveCreepCoefficient:
        """Setup and teardown for test."""
        return Form5Dot19EffectiveCreepCoefficient(phi_inf_t0=2.0, m0_eqp=100, m0_ed=200)

    def test_evaluation(self, form_5_19: Form5Dot19EffectiveCreepCoefficient) -> None:
        """Test the evaluation of the result."""
        # Expected result, manually calculated
        manually_calculated_result = 2.0 * (100 / 200)
        assert form_5_19 == pytest.approx(manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("phi_inf_t0", "m0_eqp", "m0_ed"),
        [
            (2.0, 100, 0),
        ],
    )
    def test_raise_error_when_zero_pars_are_given(self, phi_inf_t0: float, m0_eqp: float, m0_ed: float) -> None:
        """Test zero values for m0_eqp, m0_ed."""
        with pytest.raises(LessOrEqualToZeroError):
            Form5Dot19EffectiveCreepCoefficient(phi_inf_t0=phi_inf_t0, m0_eqp=m0_eqp, m0_ed=m0_ed)

    @pytest.mark.parametrize(
        ("phi_inf_t0", "m0_eqp", "m0_ed", "expected"),
        [
            (2.0, 100, 200, 1.0),
            (1.5, 150, 300, 0.75),
        ],
    )
    def test_properties(self, phi_inf_t0: float, m0_eqp: float, m0_ed: float, expected: float) -> None:
        """Test the properties of the formula."""
        form_5_19 = Form5Dot19EffectiveCreepCoefficient(phi_inf_t0=phi_inf_t0, m0_eqp=m0_eqp, m0_ed=m0_ed)
        assert form_5_19 == pytest.approx(expected, rel=1e-4)

    def test_latex(self, form_5_19: Form5Dot19EffectiveCreepCoefficient) -> None:
        """Test the latex representation of the formula."""
        # Object to test
        form_5_19_latex = form_5_19.latex()
        expected_complete = r"\phi_{ef} = \phi (\infty,t_0) \cdot \frac{M_{0,Eqp}}{M_{0,Ed}} = 2.0 \cdot \frac{100}{200} = 1.000"
        expected_short = r"\phi_{ef} = 1.000"

        assert form_5_19_latex.complete == expected_complete, "Complete representation failed."
        assert form_5_19_latex.short == expected_short, "Short representation failed."



"""Testing formula 6.4 of EN 1993-1-1+C2+A1:2005."""

import pytest

from blueprints.codes.eurocode.en_1993_1_1_2005.chapter_6_ultimate_limit_state.formula_6_4 import Form6Dot4AdditionalMoment
from blueprints.validations import NegativeValueError


class TestForm6Dot4AdditionalMoment:
    """Validation for formula 6.4 from EN 1993-1-1:2005."""

    def test_evaluation(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        n_ed = 1000.0
        e_n = 50.0

        # Object to test
        formula = Form6Dot4AdditionalMoment(n_ed=n_ed, e_n=e_n)

        # Expected result, manually calculated
        manually_calculated_result = 50000.0  # Nmm

        assert formula == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("n_ed", "e_n"),
        [
            (-1000.0, 50.0),  # n_ed is negative
            (1000.0, -50.0),  # e_n is negative
        ],
    )
    def test_raise_error_when_invalid_values_are_given(self, n_ed: float, e_n: float) -> None:
        """Test invalid values."""
        with pytest.raises(NegativeValueError):
            Form6Dot4AdditionalMoment(n_ed=n_ed, e_n=e_n)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"\Delta M_{Ed} = N_{Ed} \cdot e_{N} = 1000.000 \cdot 50.000 = 50000.000 \ Nmm",
            ),
            ("short", r"\Delta M_{Ed} = 50000.000 \ Nmm"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        n_ed = 1000.0
        e_n = 50.0

        # Object to test
        latex = Form6Dot4AdditionalMoment(n_ed=n_ed, e_n=e_n).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."



"""Testing formula 6.1 of EN 1993-1-1:2005."""

import pytest

from blueprints.codes.eurocode.en_1993_1_1_2005.chapter_6_ultimate_limit_state.formula_6_1 import Form6Dot1ElasticVerification
from blueprints.validations import LessOrEqualToZeroError, NegativeValueError


class TestForm6Dot1ElasticVerification:
    """Validation for formula 6.1 from EN 1993-1-1+C2+A1:2005."""

    def test_evaluation(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        sigma_x_ed = 100.0  # MPa
        sigma_z_ed = 50.0  # MPa
        tau_ed = 30.0  # MPa
        f_y = 355.0  # MPa
        gamma_m0 = 1.0  # dimensionless

        # Object to test
        formula = Form6Dot1ElasticVerification(
            sigma_x_ed=sigma_x_ed,
            sigma_z_ed=sigma_z_ed,
            tau_ed=tau_ed,
            f_y=f_y,
            gamma_m0=gamma_m0,
        )

        # Expected result, manually calculated
        expected_result = True

        assert formula == expected_result

    @pytest.mark.parametrize(
        ("sigma_x_ed", "sigma_z_ed", "tau_ed", "f_y", "gamma_m0"),
        [
            (-100.0, 50.0, 30.0, 355.0, 1.0),  # sigma_x_ed is negative
            (100.0, -50.0, 30.0, 355.0, 1.0),  # sigma_z_ed is negative
            (100.0, 50.0, -30.0, 355.0, 1.0),  # tau_ed is negative
            (100.0, 50.0, 30.0, -355.0, 1.0),  # f_y is negative
            (100.0, 50.0, 30.0, 355.0, -1.0),  # gamma_m0 is negative
            (100.0, 50.0, 30.0, 355.0, 0.0),  # gamma_m0 is zero
            (100.0, 50.0, 30.0, 0.0, 1.0),  # f_y is zero
        ],
    )
    def test_raise_error_when_invalid_values_are_given(
        self, sigma_x_ed: float, sigma_z_ed: float, tau_ed: float, f_y: float, gamma_m0: float
    ) -> None:
        """Test invalid values."""
        with pytest.raises((NegativeValueError, LessOrEqualToZeroError)):
            Form6Dot1ElasticVerification(
                sigma_x_ed=sigma_x_ed,
                sigma_z_ed=sigma_z_ed,
                tau_ed=tau_ed,
                f_y=f_y,
                gamma_m0=gamma_m0,
            )

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"CHECK \to \left( \frac{\sigma_{x,\text{Ed}}}{f_y / \gamma_{M0}} \right)^2 + "
                r"\left( \frac{\sigma_{z,\text{Ed}}}{f_y / \gamma_{M0}} \right)^2 - "
                r"\left( \frac{\sigma_{x,\text{Ed}}}{f_y / \gamma_{M0}} \right) "
                r"\left( \frac{\sigma_{z,\text{Ed}}}{f_y / \gamma_{M0}} \right) "
                r"+ 3 \left( \frac{\tau_{\text{Ed}}}{f_y / \gamma_{M0}} \right)^2 \leq 1 \to "
                r"\left( \frac{100.000}{355.000 / 1.000} \right)^2 + "
                r"\left( \frac{50.000}{355.000 / 1.000} \right)^2 - "
                r"\left( \frac{100.000}{355.000 / 1.000} \right) \left( \frac{50.000}{355.000 / 1.000} \right) "
                r"+ 3 \left( \frac{30.000}{355.000 / 1.000} \right)^2 \leq 1 \to OK",
            ),
            ("short", r"CHECK \to OK"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        sigma_x_ed = 100.0  # MPa
        sigma_z_ed = 50.0  # MPa
        tau_ed = 30.0  # MPa
        f_y = 355.0  # MPa
        gamma_m0 = 1.0  # dimensionless

        # Object to test
        latex = Form6Dot1ElasticVerification(
            sigma_x_ed=sigma_x_ed,
            sigma_z_ed=sigma_z_ed,
            tau_ed=tau_ed,
            f_y=f_y,
            gamma_m0=gamma_m0,
        ).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."



"""Test Concrete material from table 3.1 of EN 1992-1-1:2004."""

import pytest

from blueprints.materials.concrete import ConcreteMaterial, ConcreteStrengthClass


class TestConcreteMaterial:
    """Test class for the concrete material object."""

    def test_default_name(self, fixture_concrete_material_c30_37: ConcreteMaterial) -> None:
        """Tests the name."""
        assert fixture_concrete_material_c30_37.name == "C30/37"

    def test_name(self) -> None:
        """Tests the name."""
        concrete_with_given_name = ConcreteMaterial(
            concrete_class=ConcreteStrengthClass.C30_37,
            custom_name="C30/37 with a different name",
        )
        assert concrete_with_given_name.name == "C30/37 with a different name"

    def test_e_c(self, fixture_concrete_material_c30_37: ConcreteMaterial) -> None:
        """Tests the E_c property."""
        assert fixture_concrete_material_c30_37.e_c == 32836

    def test_custom_given_e_c(self) -> None:
        """Tests the custom given E_c."""
        concrete_with_given_e_c = ConcreteMaterial(
            concrete_class=ConcreteStrengthClass.C30_37,
            custom_e_c=30000,
        )
        assert concrete_with_given_e_c.e_c == 30000

    def test_custom_e_c_present_is_not_present(self, fixture_concrete_material_c30_37: ConcreteMaterial) -> None:
        """Tests the custom E_c."""
        assert not fixture_concrete_material_c30_37.custom_e_c_present

    def test_custom_e_c_present_is_present(self) -> None:
        """Tests the custom E_c."""
        concrete_with_given_e_c = ConcreteMaterial(
            concrete_class=ConcreteStrengthClass.C30_37,
            custom_e_c=30000,
        )
        assert concrete_with_given_e_c.custom_e_c_present

    def test_f_ck(self, fixture_concrete_material_c30_37: ConcreteMaterial) -> None:
        """Tests the f_ck property."""
        assert fixture_concrete_material_c30_37.f_ck == 30

    def test_f_ck_raises_value_error(self) -> None:
        """Tests the f_ck property raises a ValueError when the ConcreteStrengthClass is invalid."""
        invalid_concrete_strength_class = type("InvalidConcreteStrengthClass", (), {"value": "invalid_value"})()
        invalid_concrete_material = ConcreteMaterial(invalid_concrete_strength_class)
        with pytest.raises(ValueError):
            invalid_concrete_material.f_ck

    def test_f_ck_cube(self, fixture_concrete_material_c30_37: ConcreteMaterial) -> None:
        """Tests the f_ck_cube property."""
        assert fixture_concrete_material_c30_37.f_ck_cube == 37

    def test_f_ck_cube_raises_value_error(self) -> None:
        """Tests the f_ck_cube property raises a ValueError when the ConcreteStrengthClass is invalid."""
        invalid_concrete_strength_class = type("InvalidConcreteStrengthClass", (), {"value": "invalid_value"})()
        invalid_concrete_material = ConcreteMaterial(invalid_concrete_strength_class)
        with pytest.raises(ValueError):
            invalid_concrete_material.f_ck_cube

    def test_f_cd(self, fixture_concrete_material_c30_37: ConcreteMaterial) -> None:
        """Tests the f_cd property."""
        assert fixture_concrete_material_c30_37.f_cd == 20.0

    def test_f_cm(self, fixture_concrete_material_c30_37: ConcreteMaterial) -> None:
        """Tests the f_cm property."""
        assert fixture_concrete_material_c30_37.f_cm == 38.0

    def test_f_cm_cube(self, fixture_concrete_material_c30_37: ConcreteMaterial) -> None:
        """Tests the f_cm_cube property."""
        assert fixture_concrete_material_c30_37.f_cm_cube == 45.0

    def test_f_ctm_low_concrete(self, fixture_concrete_material_c30_37: ConcreteMaterial) -> None:
        """Tests the f_ctm for low concrete."""
        assert fixture_concrete_material_c30_37.f_ctm == 2.896468153816889

    def test_f_ctm_high_concrete(self, fixture_concrete_material_c90_105: ConcreteMaterial) -> None:
        """Tests the f_ctm for high concrete."""
        assert fixture_concrete_material_c90_105.f_ctm == 5.044637804355969

    def test_sigma_cr(self, fixture_concrete_material_c30_37: ConcreteMaterial) -> None:
        """Tests the sigma_cr."""
        assert fixture_concrete_material_c30_37.sigma_cr == 1.7378808922901334

    def test_strain_cr(self, fixture_concrete_material_c30_37: ConcreteMaterial) -> None:
        """Tests the strain_cr."""
        assert fixture_concrete_material_c30_37.strain_cr == 5.2926083941105295e-05

    def test_f_ctk_0_05(self, fixture_concrete_material_c30_37: ConcreteMaterial) -> None:
        """Tests the f_ctk_0_05 property."""
        assert fixture_concrete_material_c30_37.f_ctk_0_05 == 2.027527707671822

    def test_f_ctd(self, fixture_concrete_material_c30_37: ConcreteMaterial) -> None:
        """Tests the f_ctd property."""
        assert fixture_concrete_material_c30_37.f_ctd == 1.3516851384478814

    def test_f_ctk_0_95(self, fixture_concrete_material_c30_37: ConcreteMaterial) -> None:
        """Tests the f_ctk_0_95 property."""
        assert fixture_concrete_material_c30_37.f_ctk_0_95 == 3.765408599961956

    def test_e_cm(self, fixture_concrete_material_c30_37: ConcreteMaterial) -> None:
        """Tests the e_cm property."""
        assert fixture_concrete_material_c30_37.e_cm == 32836

    def test_eps_c1(self, fixture_concrete_material_c30_37: ConcreteMaterial) -> None:
        """Tests the eps_c1 property."""
        assert fixture_concrete_material_c30_37.eps_c1 == 2.1618768697354804

    def test_eps_cu1_low_concrete(self, fixture_concrete_material_c30_37: ConcreteMaterial) -> None:
        """Tests the eps_cu1."""
        assert fixture_concrete_material_c30_37.eps_cu1 == 3.5

    def test_eps_cu1_high_concrete(self, fixture_concrete_material_c90_105: ConcreteMaterial) -> None:
        """Tests the eps_cu1."""
        assert fixture_concrete_material_c90_105.eps_cu1 == 2.8

    def test_eps_c2_low_concrete(self, fixture_concrete_material_c30_37: ConcreteMaterial) -> None:
        """Tests the eps_c2."""
        assert fixture_concrete_material_c30_37.eps_c2 == 2.0

    def test_eps_c2_high_concrete(self, fixture_concrete_material_c90_105: ConcreteMaterial) -> None:
        """Tests the eps_c2."""
        assert fixture_concrete_material_c90_105.eps_c2 == 2.6004968327615767

    def test_eps_cu2_low_concrete(self, fixture_concrete_material_c30_37: ConcreteMaterial) -> None:
        """Tests the eps_cu2 property."""
        assert fixture_concrete_material_c30_37.eps_cu2 == 3.5

    def test_eps_cu2_high_concrete(self, fixture_concrete_material_c90_105: ConcreteMaterial) -> None:
        """Tests the eps_cu2 property."""
        assert fixture_concrete_material_c90_105.eps_cu2 == 2.6

    def test_n_factor_low_concrete(self, fixture_concrete_material_c30_37: ConcreteMaterial) -> None:
        """Tests the n_factor property."""
        assert fixture_concrete_material_c30_37.n_factor == 2.0

    def test_n_factor_high_concrete(self, fixture_concrete_material_c90_105: ConcreteMaterial) -> None:
        """Tests the n_factor property."""
        assert fixture_concrete_material_c90_105.n_factor == 1.4

    def test_eps_c3_low_concrete(self, fixture_concrete_material_c30_37: ConcreteMaterial) -> None:
        """Tests the eps_c3 property."""
        assert fixture_concrete_material_c30_37.eps_c3 == 1.75

    def test_eps_c3_high_concrete(self, fixture_concrete_material_c90_105: ConcreteMaterial) -> None:
        """Tests the eps_c3 property."""
        assert fixture_concrete_material_c90_105.eps_c3 == 2.3

    def test_eps_cu3_low_concrete(self, fixture_concrete_material_c30_37: ConcreteMaterial) -> None:
        """Tests the eps_cu3 property."""
        assert fixture_concrete_material_c30_37.eps_cu3 == 3.5

    def test_eps_cu3_high_concrete(self, fixture_concrete_material_c90_105: ConcreteMaterial) -> None:
        """Tests the eps_cu3 property."""
        assert fixture_concrete_material_c90_105.eps_cu3 == 2.6

    def test_rho_min(self, fixture_concrete_material_c30_37: ConcreteMaterial) -> None:
        """Tests the rho_min."""
        assert fixture_concrete_material_c30_37.rho_min(f_yd=250) == 0.2583649593204665

    def test_eq_with_different_name(self, fixture_concrete_material_c30_37: ConcreteMaterial) -> None:
        """Test equality with different name."""
        assert fixture_concrete_material_c30_37 == ConcreteMaterial(
            concrete_class=ConcreteStrengthClass.C30_37,
            custom_name="different name doesn't affect equality",
        )

    def test_inequality_with_different_e_c(self, fixture_concrete_material_c30_37: ConcreteMaterial) -> None:
        """Test inequality with different E_c."""
        assert fixture_concrete_material_c30_37 != ConcreteMaterial(
            concrete_class=ConcreteStrengthClass.C30_37,
            custom_e_c=30000,
        )

    def test_inequality_with_different_concrete_class(self, fixture_concrete_material_c30_37: ConcreteMaterial) -> None:
        """Test inequality with different concrete class."""
        assert fixture_concrete_material_c30_37 != ConcreteMaterial(concrete_class=ConcreteStrengthClass.C12_15)



"""Testing formula 6.24 of EN 1993-1-1:2005."""

import pytest

from blueprints.codes.eurocode.en_1993_1_1_2005.chapter_6_ultimate_limit_state.formula_6_24 import Form6Dot24TotalTorsionalMoment
from blueprints.validations import NegativeValueError


class TestForm6Dot24TotalTorsionalMoment:
    """Validation for formula 6.24 from EN 1993-1-1:2005."""

    def test_evaluation(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        t_t_ed = 500.0
        t_w_ed = 300.0

        # Object to test
        formula = Form6Dot24TotalTorsionalMoment(t_t_ed=t_t_ed, t_w_ed=t_w_ed)

        # Expected result, manually calculated
        manually_calculated_result = 800.0  # Nmm

        assert formula == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("t_t_ed", "t_w_ed"),
        [
            (-500.0, 300.0),  # t_t_ed is negative
            (500.0, -300.0),  # t_w_ed is negative
        ],
    )
    def test_raise_error_when_invalid_values_are_given(self, t_t_ed: float, t_w_ed: float) -> None:
        """Test invalid values."""
        with pytest.raises(NegativeValueError):
            Form6Dot24TotalTorsionalMoment(t_t_ed=t_t_ed, t_w_ed=t_w_ed)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"T_{Ed} = T_{t,Ed} + T_{w,Ed} = 500.000 + 300.000 = 800.000 \ Nmm",
            ),
            ("short", r"T_{Ed} = 800.000 \ Nmm"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        t_t_ed = 500.0
        t_w_ed = 300.0

        # Object to test
        latex = Form6Dot24TotalTorsionalMoment(t_t_ed=t_t_ed, t_w_ed=t_w_ed).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."



"""Testing formula 5.30 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_5_structural_analysis.formula_5_30 import Form5Dot30TotalDesignMoment
from blueprints.validations import LessOrEqualToZeroError, NegativeValueError


class TestForm5Dot30TotalDesignMoment:
    """Validation for formula 5.30 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        m_0ed = 50.0
        n_ed = 100.0
        n_b = 200.0

        # Object to test
        formula = Form5Dot30TotalDesignMoment(m_0ed=m_0ed, n_ed=n_ed, n_b=n_b)

        # Expected result, manually calculated
        manually_calculated_result = 100.0  # kNm

        assert formula == pytest.approx(manually_calculated_result, rel=1e-4)

    def test_evaluation_at_capacity_normalforce(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        m_0ed = 50.0
        n_ed = 200.0
        n_b = 200.0

        # Object to test
        formula = Form5Dot30TotalDesignMoment(m_0ed=m_0ed, n_ed=n_ed, n_b=n_b)

        # Expected result, manually calculated
        manually_calculated_result = 1e9  # kNm

        assert formula == pytest.approx(manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("m_0ed", "n_ed", "n_b"),
        [
            (-50.0, 100.0, 200.0),  # m_0ed is negative
            (50.0, -100.0, 200.0),  # n_ed is negative
            (50.0, 100.0, -200.0),  # n_b is negative
            (50.0, 100.0, 0.0),  # n_b is zero
        ],
    )
    def test_raise_error_when_invalid_values_are_given(self, m_0ed: float, n_ed: float, n_b: float) -> None:
        """Test invalid values."""
        with pytest.raises((NegativeValueError, LessOrEqualToZeroError)):
            Form5Dot30TotalDesignMoment(m_0ed=m_0ed, n_ed=n_ed, n_b=n_b)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"M_{Ed} = \frac{M_{0Ed}}{1 - \left(\frac{N_{Ed}}{N_{B}}\right)} = "
                r"\frac{50.000}{1 - \left(\frac{100.000}{200.000}\right)} = 100.000 \ kNm",
            ),
            ("short", r"M_{Ed} = 100.000 \ kNm"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        m_0ed = 50.0
        n_ed = 100.0
        n_b = 200.0

        # Object to test
        latex = Form5Dot30TotalDesignMoment(m_0ed=m_0ed, n_ed=n_ed, n_b=n_b).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."



"""Testing formula 3.23 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_3_materials.formula_3_23 import Form3Dot23FlexuralTensileStrength


class TestForm3Dot23FlexuralTensileStrength:
    """Validation for formula 3.23 from EN 1992-1-1:2004."""

    def test_evaluation_1(self) -> None:
        """Test the evaluation of the result."""
        # Example values
        h = 305.3  # mm
        f_ctm = 23.8  # MPa

        form_3_23 = Form3Dot23FlexuralTensileStrength(h=h, f_ctm=f_ctm)

        # Expected result, manually calculated
        manually_calculated_result = 30.81386

        assert form_3_23 == pytest.approx(expected=manually_calculated_result, rel=1e-5)

    def test_evaluation_2(self) -> None:
        """Test the evaluation of the result."""
        # Example values
        h = 1000  # mm
        f_ctm = 23.8  # MPa

        form_3_23 = Form3Dot23FlexuralTensileStrength(h=h, f_ctm=f_ctm)

        # Expected result, manually calculated
        manually_calculated_result = 23.8

        assert form_3_23 == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    def test_raise_error_when_negative_h_is_given(self) -> None:
        """Test a negative value for h."""
        # Example values
        h = -1000  # mm
        f_ctm = 23.8  # MPa

        with pytest.raises(ValueError):
            Form3Dot23FlexuralTensileStrength(h=h, f_ctm=f_ctm)

    def test_raise_error_when_negative_f_ctm_is_given(self) -> None:
        """Test a negative value for f_ctm."""
        # Example values
        h = 1000  # mm
        f_ctm = -23.8  # MPa

        with pytest.raises(ValueError):
            Form3Dot23FlexuralTensileStrength(h=h, f_ctm=f_ctm)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"f_{ctm,fl} = \max \left[ (1.6 - h/1000) \cdot f_{ctm} ; f_{ctm} \right] = "
                r"\max \left[ (1.6 - 305.300/1000) \cdot 23.800 ; 23.800 \right] = 30.814",
            ),
            ("short", r"f_{ctm,fl} = 30.814"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        h = 305.3  # mm
        f_ctm = 23.8  # MPa

        # Object to test
        form_3_23_latex = Form3Dot23FlexuralTensileStrength(h=h, f_ctm=f_ctm).latex()

        actual = {"complete": form_3_23_latex.complete, "short": form_3_23_latex.short}

        assert actual[representation] == expected, f"{representation} representation failed."



"""Testing formula 5.22 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_5_structural_analysis.formula_5_22 import Form5Dot22FactorKc, Form5Dot22FactorKs
from blueprints.validations import LessOrEqualToZeroError, NegativeValueError


class TestForm5Dot22FactorKs:
    """Validation for formula 5.22 (Ks) from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Test the evaluation of the result."""
        # Example values
        rho = 0.003  # -

        # Object to test
        form_5_22_ks = Form5Dot22FactorKs(rho=rho)

        # Expected result, manually calculated
        manually_calculated_result = 1.0  # -

        assert form_5_22_ks == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    def test_raise_error_when_rho_is_less_than_0_002(self) -> None:
        """Test a value of rho less than 0.002."""
        # Example values
        rho = 0.001

        with pytest.raises(ValueError):
            Form5Dot22FactorKs(rho=rho)

    @pytest.mark.parametrize(
        ("representation", "expected_result"),
        [
            ("complete", r"K_s = 1 = 1 = 1.000"),
            ("short", "K_s = 1.000"),
        ],
    )
    def test_latex(self, representation: str, expected_result: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        rho = 0.003  # -

        # Object to test
        form_5_22_ks_latex = Form5Dot22FactorKs(rho=rho).latex()

        actual = {
            "complete": form_5_22_ks_latex.complete,
            "short": form_5_22_ks_latex.short,
        }

        assert actual[representation] == expected_result, f"{representation} representation failed."


class TestForm5Dot22FactorKc:
    """Validation for formula 5.22 (Kc) from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Test the evaluation of the result."""
        # Example values
        k1 = 0.8  # -
        k2 = 0.9  # -
        phi_ef = 2.0  # -
        rho = 0.003  # -

        # Object to test
        form_5_22_kc = Form5Dot22FactorKc(k1=k1, k2=k2, phi_ef=phi_ef, rho=rho)

        # Expected result, manually calculated
        manually_calculated_result = 0.24  # -

        assert form_5_22_kc == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    def test_raise_error_when_rho_is_less_than_0_002(self) -> None:
        """Test a value of rho less than 0.002."""
        # Example values
        rho = 0.001
        k1 = 0.8
        k2 = 0.9
        phi_ef = 2.0

        with pytest.raises(ValueError):
            Form5Dot22FactorKc(k1=k1, k2=k2, phi_ef=phi_ef, rho=rho)

    def test_raise_error_when_phi_ef_is_less_or_equal_to_zero(self) -> None:
        """Test a value of phi_ef less or equal to zero."""
        # Example values
        rho = 0.003
        k1 = 0.8
        k2 = 0.9
        phi_ef = 0.0

        with pytest.raises(LessOrEqualToZeroError):
            Form5Dot22FactorKc(k1=k1, k2=k2, phi_ef=phi_ef, rho=rho)

    def test_raise_error_when_k1_is_negative(self) -> None:
        """Test a negative value for k1."""
        # Example values
        rho = 0.003
        k1 = -0.8
        k2 = 0.9
        phi_ef = 2.0

        with pytest.raises(NegativeValueError):
            Form5Dot22FactorKc(k1=k1, k2=k2, phi_ef=phi_ef, rho=rho)

    def test_raise_error_when_k2_is_negative(self) -> None:
        """Test a negative value for k2."""
        # Example values
        rho = 0.003
        k1 = 0.8
        k2 = -0.9
        phi_ef = 2.0

        with pytest.raises(NegativeValueError):
            Form5Dot22FactorKc(k1=k1, k2=k2, phi_ef=phi_ef, rho=rho)

    @pytest.mark.parametrize(
        ("representation", "expected_result"),
        [
            ("complete", r"K_c = \frac{k_1 \cdot k_2}{1 + \phi_{ef}} = \frac{0.800 \cdot 0.900}{1 + 2.000} = 0.240"),
            ("short", r"K_c = 0.240"),
        ],
    )
    def test_latex(self, representation: str, expected_result: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        k1 = 0.8  # -
        k2 = 0.9  # -
        phi_ef = 2.0  # -
        rho = 0.003  # -

        # Object to test
        form_5_22_kc_latex = Form5Dot22FactorKc(k1=k1, k2=k2, phi_ef=phi_ef, rho=rho).latex()

        actual = {
            "complete": form_5_22_kc_latex.complete,
            "short": form_5_22_kc_latex.short,
        }

        assert actual[representation] == expected_result, f"{representation} representation failed."



"""Testing formula 6.20 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_6_ultimate_limit_state.formula_6_20 import Form6Dot20LongitudinalShearStress
from blueprints.validations import LessOrEqualToZeroError, NegativeValueError


class TestForm6Dot20LongitudinalShearStress:
    """Validation for formula 6.20 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        delta_f_d = 100000.0  # N
        h_f = 200.0  # mm
        delta_x = 500.0  # mm

        # Object to test
        formula = Form6Dot20LongitudinalShearStress(delta_f_d=delta_f_d, h_f=h_f, delta_x=delta_x)

        # Expected result, manually calculated
        manually_calculated_result = 1.0  # MPa

        assert formula == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("delta_f_d", "h_f", "delta_x"),
        [
            (-100000.0, 200.0, 500.0),  # delta_f_d is negative
            (100000.0, -200.0, 500.0),  # h_f is negative
            (100000.0, 200.0, -500.0),  # delta_x is negative
            (100000.0, 200.0, 0.0),  # delta_x is zero
            (100000.0, 0.0, 500.0),  # h_f is zero
        ],
    )
    def test_raise_error_when_invalid_values_are_given(self, delta_f_d: float, h_f: float, delta_x: float) -> None:
        """Test invalid values."""
        with pytest.raises((NegativeValueError, LessOrEqualToZeroError)):
            Form6Dot20LongitudinalShearStress(delta_f_d=delta_f_d, h_f=h_f, delta_x=delta_x)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"v_{Ed} = \frac{\Delta F_{d}}{h_{f} \cdot \Delta x} = \frac{100000.000}{200.000 \cdot 500.000} = 1.000 \ MPa",
            ),
            ("short", r"v_{Ed} = 1.000 \ MPa"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        delta_f_d = 100000.0  # N
        h_f = 200.0  # mm
        delta_x = 500.0  # mm

        # Object to test
        latex = Form6Dot20LongitudinalShearStress(delta_f_d=delta_f_d, h_f=h_f, delta_x=delta_x).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."



"""Testing formula 6.13 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_6_ultimate_limit_state.formula_6_13 import (
    Form6Dot13ShearResistanceInclinedReinforcement,
)
from blueprints.validations import LessOrEqualToZeroError, NegativeValueError


class TestForm6Dot13ShearResistanceInclinedReinforcement:
    """Validation for formula 6.13 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        a_sw = 100.0  # mm²
        s = 200.0  # mm
        z = 300.0  # mm
        f_ywd = 400.0  # MPa
        theta = 30.0  # degrees
        alpha = 45.0  # degrees

        # Object to test
        formula = Form6Dot13ShearResistanceInclinedReinforcement(a_sw=a_sw, s=s, z=z, f_ywd=f_ywd, theta=theta, alpha=alpha)

        # Expected result, manually calculated
        manually_calculated_result = 115911.099155  # N

        assert formula == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("a_sw", "s", "z", "f_ywd", "theta", "alpha"),
        [
            (-100.0, 200.0, 300.0, 400.0, 30.0, 45.0),  # a_sw is negative
            (100.0, -200.0, 300.0, 400.0, 30.0, 45.0),  # s is negative
            (100.0, 200.0, -300.0, 400.0, 30.0, 45.0),  # z is negative
            (100.0, 200.0, 300.0, -400.0, 30.0, 45.0),  # f_ywd is negative
            (100.0, 200.0, 300.0, 400.0, -30.0, 45.0),  # theta is negative
            (100.0, 200.0, 300.0, 400.0, 30.0, -45.0),  # alpha is negative
            (100.0, 0.0, 300.0, 400.0, 30.0, 45.0),  # s is zero
        ],
    )
    def test_raise_error_when_invalid_values_are_given(self, a_sw: float, s: float, z: float, f_ywd: float, theta: float, alpha: float) -> None:
        """Test invalid values."""
        with pytest.raises((NegativeValueError, LessOrEqualToZeroError)):
            Form6Dot13ShearResistanceInclinedReinforcement(a_sw=a_sw, s=s, z=z, f_ywd=f_ywd, theta=theta, alpha=alpha)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"V_{Rd,s} = \frac{A_{sw}}{s} \cdot z \cdot f_{ywd} \cdot \left(\cot(\theta) + \cot(\alpha)\right) \cdot \sin(\alpha) = "
                r"\frac{100.000}{200.000} \cdot 300.000 \cdot 400.000 \cdot \left(\cot(30.000) + \cot(45.000)\right) \cdot \sin(45.000) = "
                r"115911.099 \ N",
            ),
            ("short", r"V_{Rd,s} = 115911.099 \ N"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        a_sw = 100.0  # mm²
        s = 200.0  # mm
        z = 300.0  # mm
        f_ywd = 400.0  # MPa
        theta = 30.0  # degrees
        alpha = 45.0  # degrees

        # Object to test
        latex = Form6Dot13ShearResistanceInclinedReinforcement(a_sw=a_sw, s=s, z=z, f_ywd=f_ywd, theta=theta, alpha=alpha).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."



"""Hexagonal cross-section shape."""

import math
from dataclasses import dataclass

from sectionproperties.pre import Geometry
from shapely.geometry import Polygon

from blueprints.structural_sections._cross_section import CrossSection
from blueprints.type_alias import MM


@dataclass(frozen=True)
class HexagonalCrossSection(CrossSection):
    """
    Class to represent a hexagonal cross-section flat on ground, using shapely for geometric calculations.

    Parameters
    ----------
    side_length : MM
        The side length of the hexagonal cross-section [mm].
    x : MM
        The x-coordinate of the hexagon's center. Default is 0.
    y : MM
        The y-coordinate of the hexagon's center. Default is 0.
    name : str
        The name of the rectangular cross-section, default is "Hexagon".
    """

    side_length: MM
    x: MM = 0
    y: MM = 0
    name: str = "Hexagon"

    def __post_init__(self) -> None:
        """Post-initialization to validate the side length."""
        if self.side_length <= 0:
            msg = f"Side length must be a positive value, but got {self.side_length}"
            raise ValueError(msg)

    @property
    def radius(self) -> MM:
        """
        Calculate the radius of the circumscribed circle of the hexagon (farthest point from the center) [mm].

        Returns
        -------
        MM
            The radius of the circumscribed circle.
        """
        return self.side_length

    @property
    def apothem(self) -> MM:
        """
        Calculate the apothem of the hexagon (distance from center to midpoint of a side) [mm].

        Returns
        -------
        MM
            The apothem of the hexagon.
        """
        return self.side_length * math.sqrt(3) / 2

    @property
    def polygon(self) -> Polygon:
        """
        Shapely Polygon representing the hexagonal cross-section.

        Returns
        -------
        Polygon
            The shapely Polygon representing the hexagon.
        """
        angle = math.pi / 3
        points = [(self.x + self.radius * math.cos(i * angle), self.y + self.radius * math.sin(i * angle)) for i in range(6)]
        return Polygon(points)

    def geometry(
        self,
        mesh_size: MM | None = None,
    ) -> Geometry:
        """Return the geometry of the hexagon cross-section.

        Properties
        ----------
        mesh_size : MM
            Maximum mesh element area to be used within
            the Geometry-object finite-element mesh. If not provided, a default value will be used.

        """
        if mesh_size is None:
            minimum_mesh_size = 2.0
            mesh_length = max(self.side_length / 10, minimum_mesh_size)
            mesh_size = mesh_length**2

        hexagon = Geometry(geom=self.polygon)
        hexagon.create_mesh(mesh_sizes=mesh_size)
        return hexagon



"""Formula 5.12N from EN 1992-1-1:2004: Chapter 5 Structural Analysis."""

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula, latex_fraction
from blueprints.type_alias import DIMENSIONLESS, KN, KNM, M
from blueprints.validations import raise_if_less_or_equal_to_zero


class Form5Dot12nRatioDistancePointZeroAndMaxMoment(Formula):
    """Class representing formula 5.12N for the calculation of the lambda ratio.

    Note:
    Ratio of the distance between point of zero and maximum moment after redistribution and
    effective depth, d. [$λ$]
    """

    label = "5.12N"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        m_sd: KNM,
        v_sd: KN,
        d: M,
    ) -> None:
        r"""[$λ$] ratio of the distance between point of zero and maximum moment after redistribution and
        effective depth, d [$-$].

        EN 1992-1-1:2004 art.5.6.3(4) - Formula (5.12N)

        Parameters
        ----------
        m_sd : KNM
            [$M_{sd}$] Design moment at the section [$kNm$].
        v_sd : KN
            [$V_{sd}$] Design shear force at the section [$kN$].
        d : M
            [$d$] Effective depth [$m$].
        """
        super().__init__()
        self.m_sd = m_sd
        self.v_sd = v_sd
        self.d = d

    @staticmethod
    def _evaluate(
        m_sd: KNM,
        v_sd: KN,
        d: M,
    ) -> DIMENSIONLESS:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_less_or_equal_to_zero(
            m_sd=m_sd,
            v_sd=v_sd,
            d=d,
        )
        return m_sd / (v_sd * d)

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 5.12N."""
        numerator = f"{self.m_sd:.{n}f}"
        denominator = f"{self.v_sd:.{n}f} \\cdot {self.d:.{n}f}"
        return LatexFormula(
            return_symbol="λ",
            result=f"{self:.{n}f}",
            equation=r"\frac{M_{sd}}{V_{sd} \cdot d}",
            numeric_equation=f"{latex_fraction(numerator=numerator, denominator=denominator)}",
            comparison_operator_label="=",
        )



"""Formula 6.29 from NEN-EN 1993-1-1+A1:2016: Chapter 6 - Ultimate Limit State."""

from blueprints.codes.eurocode.en_1993_1_1_2005 import EN_1993_1_1_2005
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula, latex_replace_symbols
from blueprints.type_alias import DIMENSIONLESS, MPA
from blueprints.validations import raise_if_negative


class Form6Dot29ReducedYieldStrength(Formula):
    r"""Class representing formula 6.29 for the calculation of reduced yield strength."""

    label = "6.29"
    source_document = EN_1993_1_1_2005

    def __init__(
        self,
        rho: DIMENSIONLESS,
        f_y: MPA,
    ) -> None:
        r"""[$f_{y,red}$] Calculation of reduced yield strength [$MPa$].

        NEN-EN 1993-1-1+A1:2016 art.6.2.8(3) - Formula (6.29)

        Parameters
        ----------
        rho : DIMENSIONLESS
            [$\rho$] Reduction factor for shear force, see equation 6.29rho (dimensionless).
        f_y : MPA
            [$f_y$] Yield strength of the material [$MPa$].
        """
        super().__init__()
        self.rho = rho
        self.f_y = f_y

    @staticmethod
    def _evaluate(
        rho: DIMENSIONLESS,
        f_y: MPA,
    ) -> MPA:
        """Evaluates the formula, for more information see the __init__ method."""
        one_minus_rho = 1 - rho
        raise_if_negative(rho=rho, f_y=f_y, one_minus_rho=one_minus_rho)

        return (1 - rho) * f_y

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 6.29."""
        _equation: str = r"(1 - \rho) \cdot f_y"
        _numeric_equation: str = latex_replace_symbols(
            _equation,
            {
                r"\rho": f"{self.rho:.{n}f}",
                r"f_y": f"{self.f_y:.{n}f}",
            },
            False,
        )
        _numeric_equation_with_units: str = latex_replace_symbols(
            _equation,
            {
                r"\rho": rf"{self.rho:.{n}f}",
                r"f_y": rf"{self.f_y:.{n}f} \ MPa",
            },
            True,
        )
        return LatexFormula(
            return_symbol=r"f_{y,red}",
            result=f"{self:.{n}f}",
            equation=_equation,
            numeric_equation=_numeric_equation,
            numeric_equation_with_units=_numeric_equation_with_units,
            comparison_operator_label="=",
            unit="MPa",
        )



"""Formula 8.19 from EN 1992-1-1:2004: Chapter 8 - Detailing of Reinforcement and Prestressing Tendons."""

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula
from blueprints.type_alias import DIMENSIONLESS, M
from blueprints.validations import raise_if_less_or_equal_to_zero


class Form8Dot19DispersionLength(Formula):
    """Class representing formula 8.19 for the calculation of dispersion length [$l_{disp}$]."""

    label = "8.19"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        l_pt: M,
        d: M,
    ) -> None:
        r"""[$l_{disp}$] Dispersion length for prestressing tendons [$m$].

        EN 1992-1-1:2004 art.8.10.2.2(4) - Formula (8.19)

        Parameters
        ----------
        l_pt : M
            [$l_{disp}$] Length of prestressing tendon [$m$].
        d : M
            [$d$] Diameter of the tendon [$m$].
        """
        super().__init__()
        self.l_pt = l_pt
        self.d = d

    @staticmethod
    def _evaluate(
        l_pt: DIMENSIONLESS,
        d: M,
    ) -> DIMENSIONLESS:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_less_or_equal_to_zero(l_pt=l_pt, d=d)
        return (l_pt**2 + d**2) ** 0.5

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 8.19."""
        return LatexFormula(
            return_symbol=r"l_{disp}",
            result=f"{self:.{n}f}",
            equation=r"\sqrt{l_{pt}^2 + d^2}",
            numeric_equation=rf"\sqrt{{{self.l_pt:.{n}f}^2 + {self.d:.{n}f}^2}}",
            comparison_operator_label="=",
        )



"""Formula 6.25 from EN 1992-1-1:2004: Chapter 6 - Ultimate limit state."""

import numpy as np

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula
from blueprints.type_alias import DEG, DIMENSIONLESS, MM2, MPA
from blueprints.validations import raise_if_greater_than_90, raise_if_less_or_equal_to_zero, raise_if_negative


class Form6Dot25DesignShearResistance(Formula):
    r"""Class representing formula 6.25 for the calculation of the design shear resistance at the interface, [$v_{Rdi}$]."""

    label = "6.25"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        c: DIMENSIONLESS,
        mu: DIMENSIONLESS,
        f_ctd: MPA,
        sigma_n: MPA,
        a_s: MM2,
        a_i: MM2,
        f_yd: MPA,
        alpha: DEG,
        nu: DIMENSIONLESS,
        f_cd: MPA,
    ) -> None:
        r"""[$v_{Rdi}$] Design shear resistance at the interface [$MPa$].

        EN 1992-1-1:2004 art.6.2.5(1) - Formula (6.25)

        Parameters
        ----------
        c : DIMENSIONLESS
            [$c$] Factor which depends on the roughness of the interface, see (2) [$-$].
        mu : DIMENSIONLESS
            [$\mu$] Factor which depends on the roughness of the interface, see (2) [$-$].
        f_ctd : MPA
            [$f_{ctd}$] Design tensile strength of concrete, as defined in 3.1.6 (2)P [$MPa$].
        sigma_n : MPA
            [$\sigma_{n}$] Stress per unit area caused by the minimum external normal force across the interface [$MPa$].
        a_s : MM2
            [$A_{s}$] Area of reinforcement crossing the interface [$mm^2$].
        a_i : MM2
            [$A_{i}$] Area of the joint [$mm^2$].
        f_yd : MPA
            [$f_{yd}$] Design yield strength of reinforcement [$MPa$].
        alpha : DEG
            [$\alpha$] Angle of the interface, limited by 45° ≤ α ≤ 90° [$degrees$].
        nu : DIMENSIONLESS
            [$\nu$] Strength reduction factor [$-$].
        f_cd : MPA
            [$f_{cd}$] Design compressive strength of concrete [$MPa$].
        """
        super().__init__()
        self.c = c
        self.mu = mu
        self.f_ctd = f_ctd
        self.sigma_n = sigma_n
        self.a_s = a_s
        self.a_i = a_i
        self.f_yd = f_yd
        self.alpha = alpha
        self.nu = nu
        self.f_cd = f_cd

    @staticmethod
    def _evaluate(
        c: DIMENSIONLESS,
        mu: DIMENSIONLESS,
        f_ctd: MPA,
        sigma_n: MPA,
        a_s: MM2,
        a_i: MM2,
        f_yd: MPA,
        alpha: DEG,
        nu: DIMENSIONLESS,
        f_cd: MPA,
    ) -> MPA:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_negative(
            c=c,
            mu=mu,
            f_ctd=f_ctd,
            sigma_n=sigma_n,
            a_s=a_s,
            f_yd=f_yd,
            alpha=alpha,
            nu=nu,
            f_cd=f_cd,
        )
        raise_if_less_or_equal_to_zero(a_i=a_i)
        raise_if_greater_than_90(alpha=alpha)

        term1 = c * f_ctd + mu * sigma_n + (a_s / a_i) * f_yd * (mu * np.sin(np.deg2rad(alpha)) + np.cos(np.deg2rad(alpha)))
        term2 = 0.5 * nu * f_cd

        return min(term1, term2)

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 6.25."""
        return LatexFormula(
            return_symbol=r"v_{Rdi}",
            result=f"{self:.{n}f}",
            equation=r"\min \left( c \cdot f_{ctd} + \mu \cdot \sigma_{n} + \frac{A_{s}}{A_{i}} \cdot f_{yd} \cdot "
            r"(\mu \cdot \sin(\alpha) + \cos(\alpha)); 0.5 \cdot \nu \cdot f_{cd} \right)",
            numeric_equation=rf"\min \left( {self.c:.{n}f} \cdot {self.f_ctd:.{n}f} + {self.mu:.{n}f} \cdot {self.sigma_n:.{n}f} "
            rf"+ \frac{{{self.a_s:.{n}f}}}{{{self.a_i:.{n}f}}} \cdot {self.f_yd:.{n}f} \cdot ({self.mu:.{n}f} \cdot \sin({self.alpha:.{n}f}) "
            rf"+ \cos({self.alpha:.{n}f})); 0.5 \cdot {self.nu:.{n}f} \cdot {self.f_cd:.{n}f} \right)",
            comparison_operator_label="=",
            unit="MPa",
        )



"""Formula 6.12 from EN 1992-1-1:2004: Chapter 6 - Ultimate Limit State."""

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula
from blueprints.type_alias import DIMENSIONLESS, MM, MM2, MPA
from blueprints.validations import raise_if_less_or_equal_to_zero, raise_if_negative


class Form6Dot12CheckMaxEffectiveCrossSectionalAreaShearReinf(Formula):
    r"""Class representing formula 6.12 for checking the maximum effective cross-sectional area of the shear reinforcement."""

    label = "6.12"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        a_sw_max: MM2,
        f_ywd: MPA,
        b_w: MM,
        s: MM,
        alpha_cw: DIMENSIONLESS,
        nu_1: DIMENSIONLESS,
        f_cd: MPA,
    ) -> None:
        r"""Check the maximum effective cross-sectional area of the shear reinforcement.

        EN 1992-1-1:2004 art.6.2.3(3) - Formula (6.12)

        Parameters
        ----------
        a_sw_max : MM2
            [$A_{sw,max}$] Maximum effective cross-sectional area of the shear reinforcement [$mm^2$].
        f_ywd : MPA
            [$f_{ywd}$] Design yield strength of the shear reinforcement [$MPa$].
        b_w : MM
            [$b_{w}$] Width of the web [$mm$].
        s : MM
            [$s$] Spacing of the shear reinforcement [$mm$].
        alpha_cw : DIMENSIONLESS
            [$\alpha_{cw}$] Coefficient taking account of the state of stress in the compression chord [$-$].
        nu_1 : DIMENSIONLESS
            [$\nu_{1}$] Strength reduction factor for concrete [$-$].
        f_cd : MPA
            [$f_{cd}$] Design value of concrete compressive strength [$MPa$].
        """
        super().__init__()
        self.a_sw_max = a_sw_max
        self.f_ywd = f_ywd
        self.b_w = b_w
        self.s = s
        self.alpha_cw = alpha_cw
        self.nu_1 = nu_1
        self.f_cd = f_cd

    @staticmethod
    def _evaluate(
        a_sw_max: MM2,
        f_ywd: MPA,
        b_w: MM,
        s: MM,
        alpha_cw: DIMENSIONLESS,
        nu_1: DIMENSIONLESS,
        f_cd: MPA,
    ) -> bool:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_less_or_equal_to_zero(
            b_w=b_w,
            s=s,
        )
        raise_if_negative(a_sw_max=a_sw_max, f_ywd=f_ywd, alpha_cw=alpha_cw, nu_1=nu_1, f_cd=f_cd)

        return (a_sw_max * f_ywd / (b_w * s)) <= (0.5 * alpha_cw * nu_1 * f_cd)

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 6.12."""
        return LatexFormula(
            return_symbol=r"CHECK",
            result="OK" if self.__bool__() else "\\text{Not OK}",
            equation=r"\frac{A_{sw,max} \cdot f_{ywd}}{b_{w} \cdot s} \leq \frac{1}{2} \cdot \alpha_{cw} \cdot \nu_{1} \cdot f_{cd}",
            numeric_equation=rf"\frac{{{self.a_sw_max:.{n}f} \cdot {self.f_ywd:.{n}f}}}{{{self.b_w:.{n}f} \cdot {self.s:.{n}f}}}"
            rf" \leq \frac{{1}}{{2}} \cdot {self.alpha_cw:.{n}f} \cdot {self.nu_1:.{n}f} \cdot {self.f_cd:.{n}f}",
            comparison_operator_label="\\to",
            unit="",
        )



"""Testing formula 7.2 and 7.2sub1 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_7_serviceability_limit_state.formula_7_2 import (
    Form7Dot2StressDistributionCoefficient,
    Form7Dot2Sub1AxialForceCoefficient,
)
from blueprints.validations import NegativeValueError


class TestForm7Dot2StressDistributionCoefficient:
    """Validation for formula 7.2 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        sigma_c = 10.0
        k_1 = 1.5
        h = 500.0
        f_ct_eff = 25.0

        # Object to test
        formula = Form7Dot2StressDistributionCoefficient(sigma_c=sigma_c, k_1=k_1, h=h, f_ct_eff=f_ct_eff)

        # Expected result, manually calculated
        manually_calculated_result = 0.29333333333333333333

        assert formula == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("sigma_c", "k_1", "h", "f_ct_eff"),
        [
            (-10.0, 1.5, 500.0, 25.0),  # sigma_c is negative
            (10.0, -1.5, 500.0, 25.0),  # k_1 is negative
            (10.0, 1.5, -500.0, 25.0),  # h is negative
            (10.0, 1.5, 500.0, -25.0),  # f_ct_eff is negative
        ],
    )
    def test_raise_error_when_invalid_values_are_given(self, sigma_c: float, k_1: float, h: float, f_ct_eff: float) -> None:
        """Test invalid values."""
        with pytest.raises(NegativeValueError):
            Form7Dot2StressDistributionCoefficient(sigma_c=sigma_c, k_1=k_1, h=h, f_ct_eff=f_ct_eff)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"k_c = min\left(0.4 \cdot \left(1 - \frac{\sigma_c}{k_1 \cdot \left(\frac{ h}{min( h, 1000)}\right) "
                r"\cdot f_{ct,eff}}\right), 1\right) = "
                r"min\left(0.4 \cdot \left(1 - \frac{10.000}{1.500 \cdot \left(\frac{ 500.000}{min( 500.000, 1000)}\right) "
                r"\cdot 25.000}\right), 1\right) = 0.293 \ -",
            ),
            ("short", r"k_c = 0.293 \ -"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        sigma_c = 10.0
        k_1 = 1.5
        h = 500.0
        f_ct_eff = 25.0

        # Object to test
        latex = Form7Dot2StressDistributionCoefficient(sigma_c=sigma_c, k_1=k_1, h=h, f_ct_eff=f_ct_eff).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."


class TestForm7Dot2Sub1AxialForceCoefficient:
    """Validation for formula 7.2sub1 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        n_ed = 5.0
        h = 500.0

        # Object to test
        formula = Form7Dot2Sub1AxialForceCoefficient(n_ed=n_ed, h=h)

        # Expected result, manually calculated
        manually_calculated_result = 1.5

        assert formula == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("n_ed", "h"),
        [
            (5.0, -500.0),  # h is negative
        ],
    )
    def test_raise_error_when_invalid_values_are_given(self, n_ed: float, h: float) -> None:
        """Test invalid values."""
        with pytest.raises(NegativeValueError):
            Form7Dot2Sub1AxialForceCoefficient(n_ed=n_ed, h=h)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"k_1 = \begin{cases} 1.5 & \text{if } N_{Ed} > 0 \\ \frac{2 \cdot min(h, 1000)}{3 \cdot h} & \text{if } N_{Ed} \le 0 \end{cases} = "
                r"\begin{cases} 1.5 & \text{if } 5.000 > 0 \\ \frac{2 \cdot min(500.000, 1000)}{3 \cdot 500.000} & \text{if } 5.000 "
                r"\le 0 \end{cases} = 1.500 \ -",
            ),
            ("short", r"k_1 = 1.500 \ -"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        n_ed = 5.0
        h = 500.0

        # Object to test
        latex = Form7Dot2Sub1AxialForceCoefficient(n_ed=n_ed, h=h).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."



"""Testing formula 8.7 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_8_detailing_of_reinforcement_and_prestressing_tendons.formula_8_3 import (
    Form8Dot3RequiredAnchorageLength,
)
from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_8_detailing_of_reinforcement_and_prestressing_tendons.formula_8_7 import (
    Form8Dot7MinimumCompressionAnchorage,
)
from blueprints.validations import NegativeValueError


class TestForm8Dot7MinimumCompressionAnchorage:
    """Validation for formula 8.7 from EN 1992-1-1:2004."""

    def test_evaluation_first_term_decisive(self) -> None:
        """Test the evaluation of the result if the first term is decisive."""
        # example values
        l_b_rqd = 500  # mm
        diameter = 8  # mm
        form_8_7 = Form8Dot7MinimumCompressionAnchorage(l_b_rqd=l_b_rqd, diameter=diameter)

        manually_calculated_result = 300  # mm

        assert form_8_7 == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    def test_evaluation_second_term_decisive(self) -> None:
        """Test the evaluation of the result if the second term is decisive."""
        # example values
        l_b_rqd = 200  # mm
        diameter = 16  # mm
        form_8_7 = Form8Dot7MinimumCompressionAnchorage(l_b_rqd=l_b_rqd, diameter=diameter)

        manually_calculated_result = 160  # mm

        assert form_8_7 == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    def test_evaluation_third_term_decisive(self) -> None:
        """Test the evaluation of the result if the third term is decisive."""
        # example values
        l_b_rqd = 150  # mm
        diameter = 8  # mm

        form_8_7 = Form8Dot7MinimumCompressionAnchorage(l_b_rqd=l_b_rqd, diameter=diameter)

        manually_calculated_result = 100  # mm

        assert form_8_7 == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    def test_evaluation_negative_l_b_rqd(self) -> None:
        """Test if NegativeValueError is raised for negative l_b_rqd."""
        # example values
        l_b_rqd = -500  # mm
        diameter = 8  # mm

        with pytest.raises(NegativeValueError):
            Form8Dot7MinimumCompressionAnchorage(l_b_rqd=l_b_rqd, diameter=diameter)

    def test_evaluation_negative_diameter(self) -> None:
        """Test if NegativeValueError is raised for negative diameter."""
        # example values
        l_b_rqd = 500  # mm
        diameter = -8  # mm

        with pytest.raises(NegativeValueError):
            Form8Dot7MinimumCompressionAnchorage(l_b_rqd=l_b_rqd, diameter=diameter)

    def test_integration_with_form_8_3(self) -> None:
        """Test the integration with Form8Dot3RequiredAnchorageLength."""
        # example values
        diameter = 12  # mm
        sigma_sd = 435  # MPA
        f_bd = 2.9  # MPA
        l_b_rqd = Form8Dot3RequiredAnchorageLength(diameter=diameter, sigma_sd=sigma_sd, f_bd=f_bd)

        form_8_7 = Form8Dot7MinimumCompressionAnchorage(l_b_rqd=l_b_rqd, diameter=diameter)

        manually_calculated_result = 270  # mm

        assert form_8_7 == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("representation", "expected_result"),
        [
            (
                "complete",
                (
                    r"l_{b,min} = \max \left\{0.6 \cdot l_{b,rqd}; 10 \cdot Ø; 100 \ \text{mm}\right\} = "
                    r"\max \left\{0.6 \cdot 500.00; 10 \cdot 8; 100\right\} = 300.00"
                ),
            ),
            ("short", "l_{b,min} = 300.00"),
        ],
    )
    def test_latex(self, representation: str, expected_result: str) -> None:
        """Test the latex representation."""
        # example values
        l_b_rqd = 500
        diameter = 8
        latex = Form8Dot7MinimumCompressionAnchorage(l_b_rqd=l_b_rqd, diameter=diameter).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert actual[representation] == expected_result, f"{representation} representation failed."



"""Formula 5.3 from EN 1993-5:2007 Chapter 5 - Ultimate limit states."""

from blueprints.codes.eurocode.en_1993_5_2007 import EN_1993_5_2007
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula
from blueprints.type_alias import DIMENSIONLESS, KNM, MM3, MPA
from blueprints.unit_conversion import MM3_TO_M3, MPA_TO_KPA
from blueprints.validations import raise_if_less_or_equal_to_zero


class Form5Dot3DesignMomentResistanceClass3(Formula):
    """Class representing formula 5.3 for design moment resistance for Class 3 cross-sections."""

    label = "5.3"
    source_document = EN_1993_5_2007

    def __init__(
        self,
        beta_b: DIMENSIONLESS,
        w_el: MM3,
        f_y: MPA,
        gamma_m_0: DIMENSIONLESS,
    ) -> None:
        r"""[$M_{c,Rd}$] Calculate design moment resistance of the cross-section (class 3) in [$kNm/m$].

        EN 1993-5:2007(E) art.5.2.2(2) - Formula (5.3)

        Parameters
        ----------
        beta_b : DIMENSIONLESS
            [$\beta_{b}$] Reduction factor for the bending resistance of the cross-section in [$-$].
        w_el : MM3
            [$W_{el}$] Elastic section modulus in [$mm^3/m$].
        f_y : MPA
            [$f_{y}$] Yield strength in [$MPa$].
        gamma_m_0 : DIMENSIONLESS
            [$\gamma_{M0}$] Partial factor for material properties in [$-$].
        """
        super().__init__()
        self.beta_b = beta_b
        self.w_el = w_el
        self.f_y = f_y
        self.gamma_m_0 = gamma_m_0

    @staticmethod
    def _evaluate(
        beta_b: DIMENSIONLESS,
        w_el: MM3,
        f_y: MPA,
        gamma_m_0: DIMENSIONLESS,
    ) -> KNM:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_less_or_equal_to_zero(
            beta_b=beta_b,
            w_el=w_el,
            f_y=f_y,
            gamma_m_0=gamma_m_0,
        )
        return beta_b * (w_el * MM3_TO_M3) * (f_y * MPA_TO_KPA) / gamma_m_0

    def latex(self, n: int = 2) -> LatexFormula:
        """Returns LatexFormula object for formula 5.3."""
        return LatexFormula(
            return_symbol=r"M_{c,Rd}",
            result=f"{self:.{n}f}",
            equation=r"\beta_B W_{el} f_y / \gamma_{M0}",
            numeric_equation=rf"{self.beta_b} \cdot {self.w_el} \cdot {self.f_y} / {self.gamma_m_0} / 1000000",
            comparison_operator_label="=",
        )



"""Formula 6.73 from EN 1992-1-1:2004: Chapter 6 - Ultimate Limit State."""

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula, latex_replace_symbols
from blueprints.type_alias import DIMENSIONLESS, MPA
from blueprints.validations import raise_if_less_or_equal_to_zero, raise_if_negative


class Form6Dot73StressRatio(Formula):
    r"""Class representing formula 6.73 for the calculation of [$R_{equ}$]."""

    label = "6.73"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        e_cd_min_equ: MPA,
        e_cd_max_equ: MPA,
    ) -> None:
        r"""[$R_{equ}$] Calculation of stress ratio.

        EN 1992-1-1:2004 art.6.8.7(1) - Formula (6.73)

        Parameters
        ----------
        e_cd_min_equ : MPA
            [$E_{cd,min,equ}$] Minimum compressive stress level [$MPa$].
        e_cd_max_equ : MPA
            [$E_{cd,max,equ}$] Maximum compressive stress level [$MPa$].
        """
        super().__init__()
        self.e_cd_min_equ = e_cd_min_equ
        self.e_cd_max_equ = e_cd_max_equ

    @staticmethod
    def _evaluate(
        e_cd_min_equ: MPA,
        e_cd_max_equ: MPA,
    ) -> DIMENSIONLESS:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_negative(e_cd_min_equ=e_cd_min_equ)
        raise_if_less_or_equal_to_zero(e_cd_max_equ=e_cd_max_equ)

        return e_cd_min_equ / e_cd_max_equ

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 6.73."""
        _equation: str = r"\frac{E_{cd,min,equ}}{E_{cd,max,equ}}"
        _numeric_equation: str = latex_replace_symbols(
            _equation,
            {
                r"E_{cd,min,equ}": f"{self.e_cd_min_equ:.{n}f}",
                r"E_{cd,max,equ}": f"{self.e_cd_max_equ:.{n}f}",
            },
            False,
        )
        return LatexFormula(
            return_symbol=r"R_{equ}",
            result=f"{self:.{n}f}",
            equation=_equation,
            numeric_equation=_numeric_equation,
            comparison_operator_label="=",
            unit="",
        )



"""Testing formula 6.24 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_6_ultimate_limit_state.formula_6_24 import Form6Dot24DesignShearStress
from blueprints.validations import LessOrEqualToZeroError, NegativeValueError


class TestForm6Dot24DesignShearStress:
    """Validation for formula 6.24 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        beta = 0.8
        v_ed = 100000.0
        z = 300.0
        b_i = 200.0

        # Object to test
        formula = Form6Dot24DesignShearStress(beta=beta, v_ed=v_ed, z=z, b_i=b_i)

        # Expected result, manually calculated
        manually_calculated_result = 4.0 / 3.0  # MPa

        assert formula == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("beta", "v_ed", "z", "b_i"),
        [
            (-0.8, 100000.0, 300.0, 200.0),  # beta is negative
            (0.8, -100000.0, 300.0, 200.0),  # v_ed is negative
            (0.8, 100000.0, -300.0, 200.0),  # z is negative
            (0.8, 100000.0, 300.0, -200.0),  # b_i is negative
            (0.8, 100000.0, 0.0, 200.0),  # z is zero
            (0.8, 100000.0, 300.0, 0.0),  # b_i is zero
        ],
    )
    def test_raise_error_when_invalid_values_are_given(self, beta: float, v_ed: float, z: float, b_i: float) -> None:
        """Test invalid values."""
        with pytest.raises((NegativeValueError, LessOrEqualToZeroError)):
            Form6Dot24DesignShearStress(beta=beta, v_ed=v_ed, z=z, b_i=b_i)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"v_{Edi} = \beta \cdot \frac{V_{Ed}}{z \cdot b_{i}} = 0.800 \cdot \frac{100000.000}{300.000 \cdot 200.000} = 1.333 \ MPa",
            ),
            ("short", r"v_{Edi} = 1.333 \ MPa"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        beta = 0.8
        v_ed = 100000.0
        z = 300.0
        b_i = 200.0

        # Object to test
        latex = Form6Dot24DesignShearStress(beta=beta, v_ed=v_ed, z=z, b_i=b_i).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."



"""Formula 6.11a/b/cN from EN 1992-1-1:2004: Chapter 6 - Ultimate limit state."""

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula
from blueprints.type_alias import DIMENSIONLESS, MPA
from blueprints.validations import raise_if_less_or_equal_to_zero


class Form6Dot11abcNCompressionChordCoefficient(Formula):
    r"""Class representing formula 6.11a/b/cN for the calculation of the coefficient taking account of the state of the stress in
    the compression chord.
    """

    label = "6.11a/b/cN"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        sigma_cp: MPA,
        f_cd: MPA,
    ) -> None:
        r"""[$\alpha_{cw}$] Coefficient taking account of the state of the stress in the compression chord [$-$].

        EN 1992-1-1:2004 art.6.2.3(3) - Formula (6.11.aN, 6.11.bN, and 6.11.cN)

        Parameters
        ----------
        sigma_cp : MPA
            [$\sigma_{cp}$] Mean compressive stress, measured positive, due to the design axial force [$MPa$].
        f_cd : MPA
            [$f_{cd}$] Design value of concrete compressive strength [$MPa$].
        """
        super().__init__()
        self.sigma_cp = sigma_cp
        self.f_cd = f_cd

    @staticmethod
    def _evaluate(
        sigma_cp: MPA,
        f_cd: MPA,
    ) -> DIMENSIONLESS:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_less_or_equal_to_zero(f_cd=f_cd, sigma_cp=sigma_cp)

        if 0 < sigma_cp <= 0.25 * f_cd:
            return 1 + sigma_cp / f_cd
        if 0.25 * f_cd < sigma_cp <= 0.5 * f_cd:
            return 1.25
        return 2.5 * (1 - sigma_cp / f_cd)

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 6.11a/b/cN."""
        return LatexFormula(
            return_symbol=r"\alpha_{cw}",
            result=f"{self:.{n}f}",
            equation=r"\begin{cases} 1 + \frac{\sigma_{cp}}{f_{cd}} & \text{if } 0 \lt \sigma_{cp} \leq 0.25 f_{cd} \\ "
            r"1.25 & \text{if } 0.25 f_{cd} \lt \sigma_{cp} \leq 0.5 f_{cd} \\ "
            r"2.5 \left(1 - \frac{\sigma_{cp}}{f_{cd}}\right) & \text{if } \sigma_{cp} \gt 0.5 f_{cd} \end{cases}",
            numeric_equation=rf"\begin{{cases}} 1 + \frac{{{self.sigma_cp:.{n}f}}}{{{self.f_cd:.{n}f}}} & \text{{if }} 0 \lt {self.sigma_cp:.{n}f} "
            rf"\leq 0.25 \cdot {self.f_cd:.{n}f} \\ 1.250 & \text{{if }} 0.25 \cdot {self.f_cd:.{n}f} \lt {self.sigma_cp:.{n}f} \leq "
            rf"0.5 \cdot {self.f_cd:.{n}f} \\ 2.5 \left(1 - \frac{{{self.sigma_cp:.{n}f}}}{{{self.f_cd:.{n}f}}}\right) & "
            rf"\text{{if }} {self.sigma_cp:.{n}f} \gt 0.5 \cdot {self.f_cd:.3f} \end{{cases}}",
            comparison_operator_label="=",
            unit="-",
        )



"""Tests for the LatexFormula class."""

import pytest

from blueprints.codes.latex_formula import LatexFormula, latex_fraction, latex_max_curly_brackets, latex_min_curly_brackets


@pytest.fixture
def fixture_latex_formula() -> LatexFormula:
    """Fixture for testing."""
    return LatexFormula(
        return_symbol="E",
        result="500",
        equation="mc^2",
        numeric_equation="5*10^2",
        comparison_operator_label="=",
    )


class TestLatexFormula:
    """Test for LatexFormula."""

    def test_short(self, fixture_latex_formula: LatexFormula) -> None:
        """Test the short representation."""
        # Expected result
        expected_result = "E = 500"

        assert fixture_latex_formula.short == expected_result

    def test_complete(self, fixture_latex_formula: LatexFormula) -> None:
        """Test the complete representation."""
        # Expected result
        expected_result = "E = mc^2 = 5*10^2 = 500"
        assert fixture_latex_formula.complete == expected_result

    def test_str(self, fixture_latex_formula: LatexFormula) -> None:
        """Test the string representation."""
        # Expected result
        expected_result = "E = mc^2 = 5*10^2 = 500"
        assert str(fixture_latex_formula) == expected_result


def test_latex_fraction() -> None:
    """Test the latex_fraction function."""
    # Example values
    numerator = 5.0
    denominator = 10.0

    # Expected result
    expected_result = r"\frac{5.0}{10.0}"

    assert latex_fraction(numerator=numerator, denominator=denominator) == expected_result


def test_latex_max_curly_brackets() -> None:
    """Test the latex_max_curly_brackets function."""
    result = latex_max_curly_brackets(r"a+b", r"500", r"c-d")
    assert result == r"\max \left\{a+b; 500; c-d\right\}"


def test_latex_min_curly_brackets() -> None:
    """Test the latex_max_curly_brackets function."""
    result = latex_min_curly_brackets(r"a+b", r"500", r"c-d")
    assert result == r"\min \left\{a+b; 500; c-d\right\}"



"""Testing sub-formula 4 for 8.8N of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_8_detailing_of_reinforcement_and_prestressing_tendons.formula_8_8n import (
    SubForm8Dot8nFunctionX,
)
from blueprints.validations import LessOrEqualToZeroError, NegativeValueError


class TestSubForm8Dot8nFunctionX:
    """Validation for sub-formula 8.8N from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Test the evaluation of the result."""
        # Example values
        cover = 60  # mm
        diameter_t = 16  # mm

        sub_form_8_8n_4 = SubForm8Dot8nFunctionX(cover=cover, diameter_t=diameter_t)

        # Expected result, manually calculated
        manually_result = 8.5

        assert sub_form_8_8n_4 == pytest.approx(expected=manually_result, rel=1e-4)

    def test_raise_error_when_c_is_negative(self) -> None:
        """Test if a NegativeValueError is raised when c is negative."""
        # Example values
        cover = -60  # mm
        diameter_t = 16  # mm

        with pytest.raises(NegativeValueError):
            SubForm8Dot8nFunctionX(cover=cover, diameter_t=diameter_t)

    def test_raise_error_when_diameter_t_is_negative(self) -> None:
        """Test if a LessOrEqualToZeroError is raised when diameter_t is negative."""
        # Example values
        cover = 60  # mm
        diameter_t = -16  # mm

        with pytest.raises(LessOrEqualToZeroError):
            SubForm8Dot8nFunctionX(cover=cover, diameter_t=diameter_t)

    def test_raise_error_when_diameter_t_is_zero(self) -> None:
        """Test if a LessOrEqualToZeroError is raised when diameter_t is zero."""
        # Example values
        cover = 60  # mm
        diameter_t = 0  # mm

        with pytest.raises(LessOrEqualToZeroError):
            SubForm8Dot8nFunctionX(cover=cover, diameter_t=diameter_t)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            ("complete", r"x = 2 \cdot \frac{c}{Ø_t} = 2 \cdot \frac{60.00}{16.00} = 8.50"),
            ("short", r"x = 8.50"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        cover = 60  # mm
        diameter_t = 16  # mm

        # Object to test
        sub_form_8_8n_4_latex = SubForm8Dot8nFunctionX(cover=cover, diameter_t=diameter_t).latex()

        actual = {"complete": sub_form_8_8n_4_latex.complete, "short": sub_form_8_8n_4_latex.short}

        assert actual[representation] == expected, f"{representation} representation failed."



"""Circular Hollow Section (CHS) steel profile."""

from collections.abc import Callable
from dataclasses import dataclass
from typing import Self

from matplotlib import pyplot as plt

from blueprints.materials.steel import SteelMaterial
from blueprints.structural_sections.cross_section_tube import TubeCrossSection
from blueprints.structural_sections.steel.steel_cross_sections._steel_cross_section import CombinedSteelCrossSection
from blueprints.structural_sections.steel.steel_cross_sections.plotters.general_steel_plotter import plot_shapes
from blueprints.structural_sections.steel.steel_cross_sections.standard_profiles.chs import CHS
from blueprints.structural_sections.steel.steel_element import SteelElement
from blueprints.type_alias import MM


@dataclass(kw_only=True)
class CHSSteelProfile(CombinedSteelCrossSection):
    """Representation of a Circular Hollow Section (CHS) steel profile.

    Attributes
    ----------
    steel_material : SteelMaterial
        The material properties of the steel.
    outer_diameter : MM
        The outer diameter of the CHS profile [mm].
    wall_thickness : MM
        The wall thickness of the CHS profile [mm].
    """

    steel_material: SteelMaterial
    outer_diameter: MM
    wall_thickness: MM

    def __post_init__(self) -> None:
        """Initialize the CHS profile."""
        self.inner_diameter = self.outer_diameter - 2 * self.wall_thickness

        self.chs = TubeCrossSection(
            name="Ring",
            outer_diameter=self.outer_diameter,
            inner_diameter=self.inner_diameter,
            x=0,
            y=0,
        )
        self.elements = [
            SteelElement(
                cross_section=self.chs,
                material=self.steel_material,
                nominal_thickness=self.wall_thickness,
            )
        ]

    @classmethod
    def from_standard_profile(
        cls,
        profile: CHS,
        steel_material: SteelMaterial,
        corrosion_outside: MM = 0,
        corrosion_inside: MM = 0,
    ) -> Self:
        """Create a CHS profile from a set of standard profiles already defined in Blueprints.

        Parameters
        ----------
        profile : CHS
            Any of the standard CHS profiles defined in Blueprints.
        steel_material : SteelMaterial
            Steel material properties for the profile.
        corrosion_outside : MM, optional
            Corrosion thickness to be subtracted from the outer diameter [mm] (default: 0).
        corrosion_inside : MM, optional
            Corrosion thickness to be added to the inner diameter [mm] (default: 0).

        Returns
        -------
        CHSSteelProfile
            The adjusted CHS steel profile considering corrosion effects.
        """
        adjusted_outer_diameter = profile.diameter - 2 * corrosion_outside
        adjusted_thickness = profile.thickness - corrosion_outside - corrosion_inside

        if adjusted_thickness <= 0:
            raise ValueError("The profile has fully corroded.")

        name = profile.alias
        if corrosion_inside or corrosion_outside:
            name += (
                f" (corrosion {'' if not corrosion_inside else f'in: {corrosion_inside} mm'}"
                f"{', ' if corrosion_inside and corrosion_outside else ''}"
                f"{'' if not corrosion_outside else f'out: {corrosion_outside} mm'})"
            )

        return cls(
            outer_diameter=adjusted_outer_diameter,
            wall_thickness=adjusted_thickness,
            steel_material=steel_material,
            name=name,
        )

    def plot(self, plotter: Callable[[CombinedSteelCrossSection], plt.Figure] | None = None, *args, **kwargs) -> plt.Figure:
        """Plot the cross-section. Making use of the standard plotter.

        Parameters
        ----------
        plotter : Callable[CombinedSteelCrossSection, plt.Figure] | None
            The plotter function to use. If None, the default Blueprints plotter for steel sections is used.
        *args
            Additional arguments passed to the plotter.
        **kwargs
            Additional keyword arguments passed to the plotter.
        """
        if plotter is None:
            plotter = plot_shapes
        return plotter(
            self,
            *args,
            **kwargs,
        )



"""Testing formula 5.14 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_5_structural_analysis.formula_5_14 import Form5Dot14SlendernessRatio
from blueprints.validations import LessOrEqualToZeroError


class TestForm5Dot14TransverseForceEffectBracingSystem:
    """Validation for formula 5.14 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Test the evaluation of the result."""
        # Example values
        l_0 = 4.0  # M
        i = 2.0  # M

        # Object to test
        form_5_14 = Form5Dot14SlendernessRatio(l_0=l_0, i=i)

        # Expected result, manually calculated
        manually_calculated_result = 2.0  # -

        assert form_5_14 == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("l_0", "i"),
        [
            (-4, 2),
            (4, -2),
        ],
    )
    def test_raise_error_when_negative_theta_i_is_given(
        self,
        l_0: float,
        i: float,
    ) -> None:
        """Test negative values for theta_i, n_a and n_b."""
        with pytest.raises(LessOrEqualToZeroError):
            Form5Dot14SlendernessRatio(l_0=l_0, i=i)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"λ = \frac{l_0}{i} = \frac{4.000}{2.000} = 2.000",
            ),
            ("short", r"λ = 2.000"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        l_0 = 4.0  # M
        i = 2.0  # M

        # Object to test
        form_5_14_latex = Form5Dot14SlendernessRatio(
            l_0=l_0,
            i=i,
        ).latex()

        actual = {
            "complete": form_5_14_latex.complete,
            "short": form_5_14_latex.short,
        }

        assert actual[representation] == expected, f"{representation} representation failed."



"""Table 4.2 from NEN-EN 1992-1-1:2005+A1:2015+NB:2016+A1:2020: Chapter 4 - Durability and cover to reinforcement."""

from blueprints.codes.eurocode.nen_en_1992_1_1_a1_2020 import NEN_EN_1992_1_1_A1_2020
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula
from blueprints.type_alias import MM
from blueprints.validations import raise_if_less_or_equal_to_zero


class Table4Dot2MinimumCoverWithRegardToBond(Formula):
    """Class representing the table 4.2 for the calculation of the minimum cover [$c_{min,b}$] [$mm$] requirements with regard to bond."""

    label = "4.2"
    source_document = NEN_EN_1992_1_1_A1_2020

    def __init__(
        self,
        diameter: MM,
        nominal_max_aggregate_size_greater_than_32_mm: bool,
    ) -> None:
        r"""[$c_{min,b}$] Calculates the minimum concrete cover with regard to bond [$mm$].

        NEN-EN 1992-1-1:2005+A1:2015+NB:2016+A1:2020 art.4.4.1.2 (3) - Table (4.2)

        Parameters
        ----------
        diameter: MM
            Diameter of the reinforcement [$mm$].
            In case of bundled bars, the equivalent diameter [$Ø_{n}$] as defined in par. 8.9.1 should be used.
            Use your own implementation of this value or use the :class:`Form8Dot14EquivalentDiameterBundledBars` class.
        nominal_max_aggregate_size_greater_than_32_mm: bool
            Is the nominal maximum aggregate size greater than 32 [$mm$]?
        """
        super().__init__()
        self.diameter = diameter
        self.nominal_max_aggregate_size_greater_than_32_mm = nominal_max_aggregate_size_greater_than_32_mm

    @staticmethod
    def _evaluate(
        diameter: MM,
        nominal_max_aggregate_size_greater_than_32_mm: bool,
    ) -> MM:
        """For more detailed documentation see the class docstring."""
        raise_if_less_or_equal_to_zero(diameter=diameter)
        if not isinstance(nominal_max_aggregate_size_greater_than_32_mm, bool):
            raise TypeError("The parameter 'nominal_max_aggregate_size_greater_than_32_mm' must be a boolean.")
        return diameter + 5 * nominal_max_aggregate_size_greater_than_32_mm

    def latex(self, n: int = 0) -> LatexFormula:
        """Returns LatexFormula object for table 4.2."""
        suffix = " + 5" if self.nominal_max_aggregate_size_greater_than_32_mm else ""
        return LatexFormula(
            return_symbol=r"c_{min,b}",
            result=f"{self:.{n}f}",
            equation=r"(equivalent) rebar diameter" + suffix,
            numeric_equation=f"{self.diameter}" + suffix,
            comparison_operator_label="=",
        )



"""Formula 2.2 from EN 1993-1-1:2005: Chapter 2: Basis of design."""

from blueprints.codes.eurocode.en_1993_1_1_2005 import EN_1993_1_1_2005
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula
from blueprints.type_alias import DIMENSIONLESS
from blueprints.validations import raise_if_negative


class Form2Dot2CharacteristicValueResistance(Formula):
    """Class representing formula 2.2 for the calculation of the characteristic value of the resistance [$R_k$]."""

    label = "2.2"
    source_document = EN_1993_1_1_2005

    def __init__(self, r_d: DIMENSIONLESS, gamma_mi: DIMENSIONLESS) -> None:
        r"""[$R_k$] Characteristic value of the resistance [$kN$].

        EN 1993-1-1:2005 art.2.5(2) - Formula (2.2)

        Parameters
        ----------
        r_d : DIMENSIONLESS
            [$R_d$] Design value of the resistance according to Annex D of EN 1990 [$kN$].
        gamma_mi : DIMENSIONLESS
            [$\gamma_{Mi}$] Recommended partial factors for the resistance [$-$].
        """
        super().__init__()
        self.r_d = r_d
        self.gamma_mi = gamma_mi

    @staticmethod
    def _evaluate(r_d: DIMENSIONLESS, gamma_mi: DIMENSIONLESS) -> float:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_negative(r_d=r_d, gamma_mi=gamma_mi)
        return r_d * gamma_mi

    def latex(self, n: int = 2) -> LatexFormula:
        """Returns LatexFormula object for formula 5.10."""
        return LatexFormula(
            return_symbol=r"R_{k}",
            result=f"{self:.{n}f}",
            equation=r"R_d \cdot \gamma_{Mi}",
            numeric_equation=rf"{self.r_d:.{n}f} \cdot {self.gamma_mi:.{n}f}",
            comparison_operator_label="=",
        )



"""Formula 6.34 and 6.35 from EN 1992-1-1:2004: Chapter 6 - Ultimate Limit State."""

from math import sqrt

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula
from blueprints.type_alias import MM
from blueprints.validations import raise_if_negative


class Form6Dot34And35ContourRadiusRectangular(Formula):
    r"""Class representing formulas 6.34 and 6.35 for the calculation of the contour radius for rectangular columns with a rectangular head
    where $l_H < 2.0 h_H$.
    """

    label = "6.34 and 6.35"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        d: MM,
        c_1: MM,
        c_2: MM,
        l_h1: MM,
        l_h2: MM,
    ) -> None:
        r"""[$r_{cont}$] Contour radius [$mm$].

        EN 1992-1-1:2004 art.6.4.3(8) - Formula (6.34 and 6.35)

        Parameters
        ----------
        d : MM
            [$d$] Effective depth [$mm$].
        c_1 : MM
            [$c_{1}$] Column size in one direction [$mm$].
        c_2 : MM
            [$c_{2}$] Column size in the perpendicular direction [$mm$].
        l_h1 : MM
            [$l_{H1}$] Head size in one direction [$mm$].
        l_h2 : MM
            [$l_{H2}$] Head size in the perpendicular direction [$mm$].
        """
        super().__init__()
        self.d = d

        self.l_1 = c_1 + 2 * l_h1
        self.l_2 = c_2 + 2 * l_h2

        self.c_1, self.c_2 = (c_2, c_1) if self.l_1 > self.l_2 else (c_1, c_2)
        self.l_h1, self.l_h2 = (l_h2, l_h1) if self.l_1 > self.l_2 else (l_h1, l_h2)

    @staticmethod
    def _evaluate(
        d: MM,
        c_1: MM,
        c_2: MM,
        l_h1: MM,
        l_h2: MM,
    ) -> MM:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_negative(d=d, c_1=c_1, c_2=c_2, l_h1=l_h1, l_h2=l_h2)

        l_1 = c_1 + 2 * l_h1
        l_2 = c_2 + 2 * l_h2

        if l_1 > l_2:
            l_1, l_2 = l_2, l_1

        return min(2 * d + 0.56 * sqrt(l_1 * l_2), 2 * d + 0.69 * l_1)

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formulas 6.34 and 6.35."""
        return LatexFormula(
            return_symbol=r"r_{cont}",
            result=f"{self:.{n}f}",
            equation=r"min\left(2 \cdot d + 0.56 \cdot \sqrt{(c_1 + 2 \cdot l_{H1}) \cdot (c_2 + 2 \cdot l_{H2})}, "
            r"2 \cdot d + 0.69 \cdot (c_1 + 2 \cdot l_{H1})\right)",
            numeric_equation=rf"min\left(2 \cdot {self.d:.{n}f} + 0.56 \cdot \sqrt{{({self.c_1:.{n}f} + 2 \cdot {self.l_h1:.{n}f}) \cdot "
            rf"({self.c_2:.{n}f} + 2 \cdot {self.l_h2:.{n}f})}}, 2 \cdot {self.d:.{n}f} + 0.69 \cdot ({self.c_1:.{n}f} + 2 \cdot {self.l_h1:.{n}f})\right)",  # noqa: E501
            comparison_operator_label="=",
            unit="mm",
        )



"""Formula 7.10 from EN 1992-1-1:2004: Chapter 7 - Serviceability Limit State."""

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula, latex_replace_symbols
from blueprints.type_alias import DIMENSIONLESS, MM2
from blueprints.validations import raise_if_less_or_equal_to_zero, raise_if_negative


class Form7Dot10RhoPEff(Formula):
    r"""Class representing formula 7.10 for the calculation of [$\rho_{p,eff}$]."""

    label = "7.10"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        a_s: MM2,
        xi_1: DIMENSIONLESS,
        a_p_prime: MM2,
        a_c_eff: MM2,
    ) -> None:
        r"""[$\rho_{p,eff}$] Calculation of the effective reinforcement ratio [$mm^2$].

        EN 1992-1-1:2004 art.7.3.2(3) - Formula (7.10)

        Parameters
        ----------
        a_s : MM2
            [$A_s$] Area of tensile reinforcement [$mm^2$].
        xi_1 : DIMENSIONLESS
            [$\xi_1$] Coefficient related to the reinforcement.
        a_p_prime : MM2
            [$A'_p$] Area of pre- or post-tensioned steel [$mm^2$].
        a_c_eff : MM2
            [$A_{c,eff}$] Effective area of concrete [$mm^2$].
        """
        super().__init__()
        self.a_s = a_s
        self.xi_1 = xi_1
        self.a_p_prime = a_p_prime
        self.a_c_eff = a_c_eff

    @staticmethod
    def _evaluate(
        a_s: MM2,
        xi_1: DIMENSIONLESS,
        a_p_prime: MM2,
        a_c_eff: MM2,
    ) -> DIMENSIONLESS:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_negative(a_s=a_s, xi_1=xi_1, a_p_prime=a_p_prime)
        raise_if_less_or_equal_to_zero(a_c_eff=a_c_eff)

        return (a_s + xi_1 * a_p_prime) / a_c_eff

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 7.10."""
        _equation: str = r"\frac{A_s + \xi_1 \cdot A'_p}{A_{c,eff}}"
        _numeric_equation: str = latex_replace_symbols(
            _equation,
            {
                r"A_s": f"{self.a_s:.{n}f}",
                r"\xi_1": f"{self.xi_1:.{n}f}",
                r"A'_p": f"{self.a_p_prime:.{n}f}",
                r"A_{c,eff}": f"{self.a_c_eff:.{n}f}",
            },
            False,
        )
        return LatexFormula(
            return_symbol=r"\rho_{p,eff}",
            result=f"{self:.{n}f}",
            equation=_equation,
            numeric_equation=_numeric_equation,
            comparison_operator_label="=",
            unit="-",
        )



"""Testing formula 6.23 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_6_ultimate_limit_state.formula_6_23 import Form6Dot23CheckShearStressInterface
from blueprints.validations import LessOrEqualToZeroError, NegativeValueError


class TestForm6Dot23CheckShearStressInterface:
    """Validation for formula 6.23 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        v_edi = 1.0
        v_rdi = 1.5

        # Object to test
        formula = Form6Dot23CheckShearStressInterface(v_edi=v_edi, v_rdi=v_rdi)

        # Expected result, manually calculated
        expected_result = True

        assert formula == expected_result

    @pytest.mark.parametrize(
        ("v_edi", "v_rdi"),
        [
            (-1.0, 1.5),  # v_edi is negative
            (1.0, -1.5),  # v_rdi is negative
        ],
    )
    def test_raise_error_when_invalid_values_are_given(self, v_edi: float, v_rdi: float) -> None:
        """Test invalid values."""
        with pytest.raises((NegativeValueError, LessOrEqualToZeroError)):
            Form6Dot23CheckShearStressInterface(v_edi=v_edi, v_rdi=v_rdi)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"CHECK \to v_{Edi} \leq v_{Rdi} \to 1.000 \leq 1.500 \to OK",
            ),
            ("short", r"CHECK \to OK"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        v_edi = 1.0
        v_rdi = 1.5

        # Object to test
        latex = Form6Dot23CheckShearStressInterface(v_edi=v_edi, v_rdi=v_rdi).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."



"""Test for formula 2.22 from CUR 228."""

import pytest

from blueprints.codes.cur.cur_228.formula_2_22 import Form2Dot22ModulusHorizontalSubgrade
from blueprints.validations import LessOrEqualToZeroError, NegativeValueError


class TestForm2Dot22ModulusHorizontalSubgrade:
    """Validation for formula 2.22 from CUR 228."""

    def test_evaluation(self) -> None:
        """Test the evaluation of the result."""
        # Example values
        r = 0.2  # m
        e_p = 2.47  # kN/m²
        alpha = 1 / 3  # -
        form_2_22 = Form2Dot22ModulusHorizontalSubgrade(r=r, e_p=e_p, alpha=alpha)

        # Expected result, manually calculated
        manually_calculated_result = 17.00760939

        assert form_2_22 == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    def test_raise_error_when_r_is_higher_then_0_3(self) -> None:
        """Tests if an ValueError is raised when r > 0.3."""
        # Example values
        r = 0.5  # m
        e_p = 2.47  # kN/m²
        alpha = 1 / 3  # -

        with pytest.raises(ValueError):
            Form2Dot22ModulusHorizontalSubgrade(r=r, e_p=e_p, alpha=alpha)

    @pytest.mark.parametrize(
        ("r", "alpha"),
        [
            (-0.6, -0.33),
            (0.2, -0.33),
            (0.2, 0.0),
            (0.0, 0.0),
        ],
    )
    def test_raise_error_when_invalid_values_are_given(self, r: float, alpha: float) -> None:
        """Test invalid values."""
        with pytest.raises(LessOrEqualToZeroError):
            Form2Dot22ModulusHorizontalSubgrade(r=r, e_p=500, alpha=alpha)

    @pytest.mark.parametrize(
        ("e_p"),
        [
            (-500.0),
        ],
    )
    def test_raise_error_when_negative_values_are_given(self, e_p: float) -> None:
        """Test invalid values."""
        with pytest.raises(NegativeValueError):
            Form2Dot22ModulusHorizontalSubgrade(r=0.2, e_p=e_p, alpha=0.33)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                (
                    "k_{h} = \\frac{2 \\cdot R}{E_{p}} \\cdot \\frac{4 \\cdot 2.65^{\\alpha} + 3 "
                    "\\alpha}{18} = \\frac{2 \\cdot 0.20}{2.47} \\cdot \\frac{4 \\cdot "
                    "2.65^{0.33} + 3 \\cdot 0.33}{18} = 17.01 \\ kN/m^3"
                ),
            ),
            ("short", r"k_{h} = 17.01 \ kN/m^3"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        r = 0.2  # m
        e_p = 2.47  # kN/m²
        alpha = 1 / 3  # -

        # Object to test
        form_2_1_a_latex = Form2Dot22ModulusHorizontalSubgrade(r=r, e_p=e_p, alpha=alpha).latex()

        actual = {"complete": form_2_1_a_latex.complete, "short": form_2_1_a_latex.short}

        assert actual[representation] == expected, f"{representation} representation failed."



"""Testing formula 6.32 of EN 1993-1-1:2005."""

import pytest

from blueprints.codes.eurocode.en_1993_1_1_2005.chapter_6_ultimate_limit_state.formula_6_32 import Form6Dot32MNrdRectangular
from blueprints.validations import LessOrEqualToZeroError, NegativeValueError


class TestForm6Dot32MNrdRectangular:
    """Validation for formula 6.32 from EN 1993-1-1:2005."""

    def test_evaluation(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        m_pl_rd = 5000.0  # Nmm
        n_ed = 1000.0  # N
        n_pl_rd = 2000.0  # N

        # Object to test
        formula = Form6Dot32MNrdRectangular(m_pl_rd=m_pl_rd, n_ed=n_ed, n_pl_rd=n_pl_rd)

        # Expected result, manually calculated
        manually_calculated_result = 3750.0  # Nmm

        assert formula == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("m_pl_rd", "n_ed", "n_pl_rd"),
        [
            (-5000.0, 1000.0, 2000.0),  # m_pl_rd is negative
            (5000.0, -1000.0, 2000.0),  # n_ed is negative
            (5000.0, 1000.0, -2000.0),  # n_pl_rd is negative
        ],
    )
    def test_raise_error_when_negative_values_are_given(self, m_pl_rd: float, n_ed: float, n_pl_rd: float) -> None:
        """Test invalid negative values."""
        with pytest.raises((NegativeValueError, LessOrEqualToZeroError)):
            Form6Dot32MNrdRectangular(m_pl_rd=m_pl_rd, n_ed=n_ed, n_pl_rd=n_pl_rd)

    @pytest.mark.parametrize(
        ("m_pl_rd", "n_ed", "n_pl_rd"),
        [
            (5000.0, 1000.0, 0.0),  # n_pl_rd is zero
        ],
    )
    def test_raise_error_when_zero_values_are_given(self, m_pl_rd: float, n_ed: float, n_pl_rd: float) -> None:
        """Test invalid zero values."""
        with pytest.raises(LessOrEqualToZeroError):
            Form6Dot32MNrdRectangular(m_pl_rd=m_pl_rd, n_ed=n_ed, n_pl_rd=n_pl_rd)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"M_{N,Rd} = M_{pl,Rd} \cdot \left[ 1 - \left( \frac{N_{Ed}}{N_{pl,Rd}} \right)^2 \right] = "
                r"5000.000 \cdot \left[ 1 - \left( \frac{1000.000}{2000.000} \right)^2 \right] = 3750.000 \ Nmm",
            ),
            (
                "complete_with_units",
                r"M_{N,Rd} = M_{pl,Rd} \cdot \left[ 1 - \left( \frac{N_{Ed}}{N_{pl,Rd}} \right)^2 \right] = "
                r"5000.000 \ Nmm \cdot \left[ 1 - \left( \frac{1000.000 \ N}{2000.000 \ N} \right)^2 \right] = 3750.000 \ Nmm",
            ),
            ("short", r"M_{N,Rd} = 3750.000 \ Nmm"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        m_pl_rd = 5000.0  # Nmm
        n_ed = 1000.0  # N
        n_pl_rd = 2000.0  # N

        # Object to test
        latex = Form6Dot32MNrdRectangular(m_pl_rd=m_pl_rd, n_ed=n_ed, n_pl_rd=n_pl_rd).latex()

        actual = {
            "complete": latex.complete,
            "complete_with_units": latex.complete_with_units,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."
